/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.area;

/**
 * @author Vity
 */

import net.wordrider.area.actions.*;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.plaf.TextUI;
import javax.swing.text.*;
import javax.swing.undo.UndoManager;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.*;
import java.awt.event.*;

public final class RiderArea extends JTextPane implements DropTargetListener, DragSourceListener,
        DragGestureListener {
    private static final int LEFT_BORDER_WIDTH = 30;
    private boolean isModified = false;
    private static boolean isOvertypeMode;
    private static InputMap defaultInputMap = null;
    private final Caret defaultCaret = getCaret();
    private static final Caret overtypeCaret = new OvertypeCaret();
    //private int leftParenthis = -1, rightParenthis = -1;

    private static final ClipboardOwner defaultClipboardOwner = new ClipboardObserver();
    private AreaUndoManager undoManager = null;


    /**
     * a data flavor for transferables processed by this component
     */
    private final static DataFlavor df =
            new DataFlavor(StyledContent.class, "StyledContent");

//    final private DropTarget dropTarget = new DropTarget(this, this);

    final private DragSource dragSource = new DragSource();

    private int lastSelStart = 0;

    private int lastSelEnd = 0;

    private int dndEventLocation = 0;

    private boolean dragStartedHere = false;


    private static int maxTI89FontWidth = 0;
    private static int maxTI92FontWidth = 0;

    public static final int FULLVIEWBORDER = 0;
    public static final int TI89VIEWBORDER = 1;
    public static final int TI92VIEWBORDER = 2;

    private int borderType = FULLVIEWBORDER;
    private int maxRightWidth = -1;
    private boolean noModifyChange = false;

    public RiderArea() {
        super();
        init();
    }

    public final RiderDocument getDoc() {
        return (RiderDocument) this.getDocument();
    }

    public final RiderEditorKit getKit() {
        return (RiderEditorKit) this.getEditorKit();
    }

    // private void updateRightInset() {
    //  ((AreaBorder) this.getBorder()).setRightInset(getWidth() - getTI89MaxTextWidth() - LEFT_BORDER_WIDTH);
    //revalidate();
    //repaint();
//        SwingUtilities.invokeLater(new Runnable() {
//            public void run() {
//                repaint();
//            }
//        });
    //  this.setBorder(new AreaBorder(2, LEFT_BORDER_WIDTH, 0, getWidth() - getTI89MaxTextWidth() - LEFT_BORDER_WIDTH));
    //}


    private final void init() {
        new DropTarget(this, this);
        if (maxTI89FontWidth == 0)
            maxTI89FontWidth = this.getFontMetrics(RiderStyles.getAreaFont()).stringWidth(RiderStyles.TI89MAXLENGTHSTRING) + 1;
        if (maxTI92FontWidth == 0)
            maxTI92FontWidth = this.getFontMetrics(RiderStyles.getAreaFont()).stringWidth(RiderStyles.TI92MAXLENGTHSTRING) + 1;
        dragSource.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_MOVE, this);
        this.setBorder(new AreaBorder(2, LEFT_BORDER_WIDTH, 0, 4, this));
        createUndoManager();
        //undoManager = new AreaUndoManager(this); // after setting of document
        if (defaultInputMap == null) {
            Utils.addKeyActions(this);
            defaultInputMap = initActions(this.getInputMap());
        }
        this.setInputMap(JComponent.WHEN_FOCUSED, defaultInputMap);

        this.setFont(RiderStyles.getAreaFont());
        final MouseListener[] listeners = this.getMouseListeners();
        for (int i = 0; i < listeners.length; ++i)
            this.removeMouseListener(listeners[i]);
        this.addMouseMotionListener(new MouseMoveAdapter());
        this.addMouseListener(new MouseClickAdapter());
        for (int i = 0; i < listeners.length; ++i)
            this.addMouseListener(listeners[i]);
        this.addKeyListener(new AlfaBetaKeyListener());
        overtypeCaret.setBlinkRate(defaultCaret.getBlinkRate());

        setOvertypeMode(false);
        //this.addCaretListener(this);
        //this.add(ContextMenu.getInstance());
    }

    boolean showRightBorder() {
        return borderType != FULLVIEWBORDER;
    }

    boolean limitRightBorder() {
        return borderType != FULLVIEWBORDER;
    }

    public final UndoManager getUndoManager() {
        return undoManager;
    }

    public final int getViewBorder() {
        return borderType;
    }

    public final void setViewBorder(final int type) {
        switch (type) {
            case FULLVIEWBORDER:
                maxRightWidth = -1;
                break;
            case TI89VIEWBORDER:
                maxRightWidth = RiderArea.maxTI89FontWidth;
                break;
            case TI92VIEWBORDER:
                maxRightWidth = RiderArea.maxTI92FontWidth;
                break;
            default :
                return;
        }
        borderType = type;
        final Component component = this;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                component.validate();
                component.repaint();
            }
        });
    }

    /*
	 *	Set the caret to use depending on overtype/insert mode
	 */
    public void setOvertypeMode(final boolean overtype) {
        isOvertypeMode = overtype;
        final int pos = getCaretPosition();
        this.putClientProperty("overtype", Boolean.valueOf(overtype));
        this.firePropertyChange("overtype", overtype, overtype);
        if (overtype)
            setCaret(overtypeCaret);
        else {
            setCaret(defaultCaret);
            getDoc().refresh(pos, 1);
        }
        setCaretPosition(pos);
    }

    int getMaxTextWidth() {
        return maxRightWidth;
    }

    private final class MouseClickAdapter extends MouseAdapter {
        public final void mouseReleased(final MouseEvent e) {
            lastSelStart = getSelectionStart();
            lastSelEnd = getSelectionEnd();
        }

        public final void mouseClicked(final MouseEvent e) {
            final RiderArea area = (RiderArea) e.getSource();
            if (isInLeftBorder(e)) {
                final int pos = area.getCaretPosition();
                final Element el = area.getBookmarkElement(e, area.viewToModel(e.getPoint()));
                if (el != null) {
                    area.getDoc().toggleBookmark(el.getStartOffset());
                    area.setCaretPosition(pos);
                }
                e.consume();
            } else {
                if (e.getButton() != MouseEvent.BUTTON1) {
                    ContextMenu.getInstance().show(area, e.getX(), e.getY());
                    e.consume();
                } else {
                    if (e.getClickCount() > 1) {
                        lastSelStart = getSelectionStart();
                        lastSelEnd = getSelectionEnd();
                    }
                }
            }
        }
    }

    public final void setDocument(final Document doc) {
//        if (this.getDocument() != null)
//            this.getDocument().removeDocumentListener(this);
        super.setDocument(doc);    //call to super
//        doc.addDocumentListener(this);
        setModified(false);
        createUndoManager();
    }

    private void createUndoManager() {
        undoManager = new AreaUndoManager(this);
    }

    public final void setModified(final boolean modified) {
        if (isModified != modified) {
            if (noModifyChange)
                return;
            this.isModified = modified;//must be first
            this.putClientProperty("modified", Boolean.valueOf(modified));
            //this.firePropertyChange("modified", !modified, modified);
        }
    }

    public final boolean isModified() {
        return this.isModified;
    }
//    public final void setAttributeSet(final AttributeSet attr,
//                                      final boolean setParagraphAttributes) {
//        final int selStart = this.getSelectionStart();
//        final int selEnd = this.getSelectionEnd();
//
//        if (setParagraphAttributes) {
//            document.setParagraphAttributes(selStart,
//                    selEnd - selStart, attr, false);
//        } else if (selStart != selEnd)
//            document.setCharacterAttributes(selStart,
//                    selEnd - selStart, attr, false);
//        else {
//            final MutableAttributeSet inputAttributes =
//                    this.getInputAttributes();
//            inputAttributes.addAttributes(attr);
//        }
//    }



    private final void replaceSelection(final StyledContent content) {
        final Caret caret = getCaret();
        try {
            final int p0 = Math.min(caret.getDot(), caret.getMark());
            final int p1 = Math.max(caret.getDot(), caret.getMark());
            if (p0 != p1) {
                this.getDocument().remove(p0, p1 - p0);
            }
            if (content != null) {
                content.insert(this, p0);
            }
        } catch (Exception e) {
            Utils.processException(e);
        }
    }


    public final void noModifyUpdate(boolean update) {
        this.noModifyChange = update;
    }

    /*
	 *  Override method from JComponent
	 */
    public void replaceSelection(String text) {
        //  Implement overtype mode by selecting the character at the current
        //  caret position

        if (isOvertypeMode()) {
            final int pos = getCaretPosition();
            if (getSelectedText() == null && pos < getDocument().getLength())
                moveCaretPosition(pos + 1);

        }
        super.replaceSelection(text);
    }

    private static boolean isOvertypeMode() {
        return isOvertypeMode;
    }

/*
	 *  Paint a horizontal line the width of a column and 1 pixel high
	 */
    private static class OvertypeCaret extends DefaultCaret {
        /*
         *  The overtype caret will simply be a horizontal line one pixel high
         *  (once we determine where to paint it)
         */
        public void paint(Graphics g) {
            if (isVisible()) {
                try {
                    final JTextComponent component = getComponent();
                    final TextUI mapper = component.getUI();
                    final Rectangle r = mapper.modelToView(component, getDot());
                    final Color caretColor = component.getCaretColor();
                    g.setColor(caretColor);
                    final int width = g.getFontMetrics().charWidth('w');
                    final int y = r.y + r.height - 2;
                    g.drawLine(r.x, y, r.x + width - 2, y);
                } catch (BadLocationException e) {
                    Utils.processException(e);
                }
            }
        }

        /*
         *  Damage must be overridden whenever the paint method is overridden
         *  (The damaged net.wordrider.area is the net.wordrider.area the caret is painted in. We must
         *  consider the net.wordrider.area for the default caret and this caret)
         */
        protected synchronized void damage(final Rectangle r) {
            if (r != null) {
                final JTextComponent component = getComponent();
                x = r.x;
                y = r.y;
                width = component.getFontMetrics(component.getFont()).charWidth('w');
                height = r.height;
                repaint();
            }
        }
    }

    protected final EditorKit createDefaultEditorKit() {
        return new RiderEditorKit();
    }

    public final void cut() {
        if (isEditable() && isEnabled()) {
            copy();
            replaceSelection("");
        }
    }

    public final void copy() {
        try {
            final StyledContent st = new StyledContent(this);
            final StyledContentSelection contents = new StyledContentSelection(st);
            final Clipboard clipboard = getToolkit().getSystemClipboard();
            clipboard.setContents(contents, defaultClipboardOwner);
        } catch (Exception e) {
            Utils.processException(e);
        }
    }

    public final void paste() {
        //this.firePropertyChange("undoredo",false, true);
        final Clipboard clipboard = getToolkit().getSystemClipboard();
        final Transferable content = clipboard.getContents(this);
        if (content != null) {
            try {
                if (content.isDataFlavorSupported(df))
                    replaceSelection((StyledContent) content.getTransferData(df));
                else if (content.isDataFlavorSupported(DataFlavor.stringFlavor)) {
                    String clipbString = content.getTransferData(DataFlavor.stringFlavor).toString();
//                    clipbString = clipbString.replaceAll("","a");
//                    clipbString = clipbString.replaceAll("","e");
//                    clipbString = clipbString.replaceAll("","i");
//                    clipbString = clipbString.replaceAll("","o");
//                    clipbString = clipbString.replaceAll("","u");
//                    clipbString = clipbString.replaceAll("","u");
//                    clipbString = clipbString.replaceAll("","A");
//                    clipbString = clipbString.replaceAll("","E");
//                    clipbString = clipbString.replaceAll("","O");
//                    clipbString = clipbString.replaceAll("","I");
//                    clipbString = clipbString.replaceAll("","U");
//                    clipbString = clipbString.replaceAll("","Y");
//                    clipbString = clipbString.replaceAll("","z");
//                    clipbString = clipbString.replaceAll("","Z");
//                    clipbString = clipbString.replaceAll("","s");
//                    clipbString = clipbString.replaceAll("","S");
//                    clipbString = clipbString.replaceAll("","e");
//                    clipbString = clipbString.replaceAll("","y");
//                    clipbString = clipbString.replaceAll("","E");
//                    clipbString = clipbString.replaceAll("","c");
//                    clipbString = clipbString.replaceAll("","C");
//                    clipbString = clipbString.replaceAll("","r");
//                    clipbString = clipbString.replaceAll("","R");
//                    clipbString = clipbString.replaceAll("","D");
//                    clipbString = clipbString.replaceAll("","T");
//                    clipbString = clipbString.replaceAll("","d");
//                    clipbString = clipbString.replaceAll("","N");
//                    clipbString = clipbString.replaceAll("","n");
                    replaceSelection(clipbString);
                }
            } catch (Exception e) {
                Utils.processException(e);
            }
        }
        //this.firePropertyChange("undoredo",true, false);
    }

//    public final int getTI89MaxTextWidth() {
//        return maxTI89FontWidth;
//    }

//    public final boolean processKeyBinding(final KeyStroke ks, final KeyEvent e, final int condition, final boolean pressed) {
//        final int caretPosition = this.getCaretPosition();
//        if (ks.getKeyEventType() == KeyEvent.KEY_PRESSED) {
//            if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
//                if (caretPosition > 0) {
//                    final Element sectionElem = document.getDefaultRootElement();
//                    final Element paragraph = sectionElem.getElement(sectionElem.getElementIndex(caretPosition));
//                    //final int elCount = paragraph.getElementCount();
//                    if (paragraph.getStartOffset() == caretPosition && paragraph.getElementCount() > 0) {
//                        final Element el = paragraph.getElement(0);
//                        if (el.getName().equals(StyleConstants.ComponentElementName)) {
//                            //StyleConstants.setComponent();
//                        }
//                    }
//                }
//            } else {
//                if (RiderStyles.isReadonlySection(document.getParagraphElement(caretPosition))) {
//                    //System.out.println("it's read only section LASTOPENFOLDER_KEY code " + e.getKeyCode());
//                    if (!e.isControlDown() && e.getKeyCode() != KeyEvent.VK_DELETE && Character.isUnicodeIdentifierPart(e.getKeyChar())) {
//                        //System.out.println("stopping a LASTOPENFOLDER_KEY");
//                        e.setKeyCode(0);
//                        return false;
//                    }
//                }
//            }
//        }
//        return super.processKeyBinding(ks, e, condition, pressed);
//    }

    public final Color getCaretColor() {
        if (this.hasFocus()) {
            final int caretPos = this.getCaretPosition();
            try {
                if (RiderStyles.isInvert(getInputAttributes()) && Utilities.getRowEnd(this, caretPos) != caretPos)
                    return Color.WHITE;
            } catch (BadLocationException e) {
                Utils.processException(e);
            }
        }
        return super.getCaretColor();
    }


    private static InputMap initActions(final InputMap map) {
        map.put(KeyStroke.getKeyStroke(KeyEvent.VK_Y, InputEvent.CTRL_MASK), RedoAction.getInstance());
        map.put(KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK), RedoAction.getInstance());
        map.put(KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE, InputEvent.ALT_MASK), UndoAction.getInstance());
        map.put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, InputEvent.CTRL_MASK), new CtrlDeleteAction());
        map.put(KeyStroke.getKeyStroke(KeyEvent.VK_0, InputEvent.ALT_MASK), AligmentLeftAction.getInstance());
        map.put(KeyStroke.getKeyStroke("F4"), RotateAligmentStyleAction.getInstance());
        map.put(KeyStroke.getKeyStroke("F5"), RotateMarginStyleAction.getInstance());
        map.put(GetNextTabAction.getInstance().getKeyStroke(), GetNextTabAction.getInstance());
        map.put(GetPrevTabAction.getInstance().getKeyStroke(), GetPrevTabAction.getInstance());
        return map;
    }

    final private class MouseMoveAdapter extends MouseMotionAdapter {

        public void mouseMoved(final MouseEvent e) {
            super.mouseMoved(e);
            areaMouseMoved(e);
        }

    }

    private static boolean isInLeftBorder(final MouseEvent e) {
        return e.getX() < LEFT_BORDER_WIDTH - (LEFT_BORDER_WIDTH - AreaBorder.SHADOW_BAR_WIDTH);
    }

//    private Component getMoveableComponent(final int pos) {
//        if (pos != -1) {
//            final Element el = this.getStyledDocument().getParagraphElement(pos);
//            return RiderStyles.isMoveableComponent(el);
//        }
//        return null;
//    }

    private Element getBookmarkElement(final MouseEvent e, final int pos) {
        if (pos != -1) {
            final Element el = this.getStyledDocument().getParagraphElement(pos);
            if (RiderStyles.isReadonlySection(el) || RiderStyles.isMath(el))
                return null;
            final Rectangle rect;
            try {
                rect = this.modelToView(el.getStartOffset());
            } catch (BadLocationException ex) {
                Utils.processException(ex);
                return null;
            }
            if (e.getY() >= rect.y + 2 && e.getY() <= rect.height + rect.y - 2)
                return el;
        }
        return null;
    }

    private static final class ClipboardObserver implements ClipboardOwner {
        public final void lostOwnership(final Clipboard clipboard, final Transferable contents) {
        }
    }

    private final class AlfaBetaKeyListener implements KeyListener {
        //GAMA for old TI92PlusPc \u0083
        private static final char ALFA = '\u20AC', GAMA = '\u0192', GAMA_U = '\u201A', BETA = '\u0081', DELTA = '\u2026', DELTA_U = '\u201E', EPSILON = '\u2020', FI = '\u2018', LAMBDA = '\u2030', MY = '\u00B1', PI = '\u0152', PI_U = '\u2039', RO = '\u008D', SIGMA = '\u008F', SIGMA_U = '\u017D', TAU = '\u0090', OMEGA = '\u201D', OMEGA_U = '\u201C', XI = '\u0160', PSI = '\u2019', ZETA = '\u2021', INTEGRAL = '\u00b7', COMPLEX_NUMBER = '\u00c6', EXPONENT = '\u2014', THETA = '\u02C6', DERIVE = '\u00B6', CALC_MINUS='\u00AA';

        private void insertChar(final KeyEvent e, final char lowerCh, final char upperCh) {
            final JTextPane pane = (JTextPane) e.getSource();
            try {
                pane.getDocument().insertString(pane.getCaretPosition(), String.valueOf((e.isShiftDown()) ? upperCh : lowerCh), pane.getInputAttributes());
            } catch (BadLocationException ex) {
                Utils.processException(ex);
            }
            e.consume();
        }

        public final void keyPressed(final KeyEvent e) {
            if (e.isAltDown()) {
                final int code = e.getKeyCode();
                if (Character.isLetter(e.getKeyChar()) || code == KeyEvent.VK_QUOTE || code == KeyEvent.VK_QUOTEDBL || code == KeyEvent.VK_MINUS)
                        {
                            char lower = 0, upper = 0;
                            switch (code) {
                                case KeyEvent.VK_A:
                                lower = upper = ALFA;
                                break;
                                case KeyEvent.VK_B:
                                lower = upper = BETA;
                                break;
                                case KeyEvent.VK_D:
                                lower = DELTA;
                                upper = DELTA_U;
                                break;
                                case KeyEvent.VK_E:
                                lower = EPSILON;
                                upper = EXPONENT;
                                break;
                                case KeyEvent.VK_F:
                                lower = upper = FI;
                                break;
                                case KeyEvent.VK_G:
                                lower = GAMA;
                                upper = GAMA_U;
                                break;
                                case KeyEvent.VK_I:
                                lower = COMPLEX_NUMBER;
                                upper = INTEGRAL;
                                break;
                                case KeyEvent.VK_L:
                                lower = upper = LAMBDA;
                                break;
                                case KeyEvent.VK_M:
                                lower = upper = MY;
                                break;
                                case KeyEvent.VK_P:
                                lower = PI;
                                upper = PI_U;
                                break;
                                case KeyEvent.VK_R:
                                lower = upper = RO;
                                break;
                                case KeyEvent.VK_S:
                                lower = SIGMA;
                                upper = SIGMA_U;
                                break;
                                case KeyEvent.VK_T:
                                lower = TAU;
                                upper = THETA;
                                break;
                                case KeyEvent.VK_W:
                                case KeyEvent.VK_O:
                                lower = OMEGA;
                                upper = OMEGA_U;
                                break;
                                case KeyEvent.VK_X:
                                lower = upper = XI;
                                break;
                                case KeyEvent.VK_Y:
                                lower = upper = PSI;
                                break;
                                case KeyEvent.VK_Z:
                                lower = upper = ZETA;
                                break;
                                case KeyEvent.VK_QUOTE:
                                case KeyEvent.VK_QUOTEDBL:
                                lower = upper = DERIVE;
                                break;
                                case KeyEvent.VK_MINUS :
                                    lower = upper = CALC_MINUS;
                                default :
                                break;
                            }

                            if (lower != 0)
                            insertChar(e, lower, upper);
                        }
            } else if (!(e.isAltDown() || e.isShiftDown() || e.isControlDown()) && e.getKeyCode() == KeyEvent.VK_INSERT)
                setOvertypeMode(!isOvertypeMode());
        }

        public final void keyReleased(final KeyEvent e) {
            //implement - call to super class
        }

        public final void keyTyped(final KeyEvent e) {
            //implement - call to super class
        }
    }

    public final void changedUpdate(final DocumentEvent e) {
        //setModified(true);
    }

    public final void insertUpdate(final DocumentEvent e) {
        setModified(true);
//        final Element paraElement = getDoc().getParagraphElement(getCaretPosition());
//        SwingUtilities.invokeLater(new Runnable() {
//            public void run() {
//
//                if (leftParenthis != -1) {
//                    markParenthis(paraElement, leftParenthis);
//                    markParenthis(paraElement, rightParenthis);
//                    leftParenthis = rightParenthis = -1;
//                }
//            }
//        });
    }

    public final void removeUpdate(final DocumentEvent e) {
        setModified(true);

    }

    private void areaMouseMoved(final MouseEvent e) {
        if (isInLeftBorder(e)) {
            final int position = ((JTextComponent) (e.getSource())).viewToModel(e.getPoint());
            if (getBookmarkElement(e, position) != null) {
                if (this.getCursor().getType() != Cursor.HAND_CURSOR)
                    this.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            } else if (this.getCursor().getType() != Cursor.DEFAULT_CURSOR) this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        } else {
//            final int position = ((JTextComponent) (e.getSource())).viewToModel(e.getPoint());
//            SeparatorLine moveable = (SeparatorLine)getMoveableComponent(position);
//            if (moveable != null && moveable.contains(e.getX(), e.getY())) {
//                if (this.getCursor().getType() != Cursor.MOVE_CURSOR) this.setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
//            } else if (this.getCursor().getType() != Cursor.TEXT_CURSOR) this.setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
            if (this.getCursor().getType() != Cursor.TEXT_CURSOR) this.setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
        }
    }


    /* ------ start of drag and drop implementation -------------------------
       (partly using example code of the Java Tutorial)
*/


    /**
     * a drag gesture has been initiated
     */
    public void dragGestureRecognized(final DragGestureEvent event) {
        final int selStart = getSelectionStart();
        //       int selEnd = getSelectionEnd();
        try {
            if ((lastSelEnd > lastSelStart) &&
                    (selStart >= lastSelStart) &&
                    (selStart < lastSelEnd)) {
                dragStartedHere = true;
                select(lastSelStart, lastSelEnd);
                final StyledContent text = new StyledContent(this);
                final StyledContentSelection trans = new StyledContentSelection(text);
                dragSource.startDrag(event, DragSource.DefaultMoveDrop, trans, this);
            }
        } catch (Exception e) {
            Utils.processException(e);
        }
    }

    /**
     * this message goes to DragSourceListener, informing it that the dragging
     * has entered the DropSite
     */
    public void dragEnter(final DragSourceDragEvent event) {
    }

    /**
     * this message goes to DragSourceListener, informing it that the dragging
     * has exited the DropSite
     */
    public void dragExit(final DragSourceEvent event) {
    }

    /**
     * this message goes to DragSourceListener, informing it that
     * the dragging is currently ocurring over the DropSite
     */
    public void dragOver(final DragSourceDragEvent event) {
    }

    /**
     * this message goes to DragSourceListener, informing it that the dragging
     * has ended
     */
    public void dragDropEnd(final DragSourceDropEvent event) {
        dragStartedHere = false;
    }

    /**
     * is invoked when the user changes the dropAction
     */
    public void dropActionChanged(final DragSourceDragEvent event) {
    }

    /**
     * is invoked when you are dragging over the DropSite
     */
    public void dragEnter(final DropTargetDragEvent event) {
    }

    /**
     * is invoked when you are exit the DropSite without dropping
     */
    public void dragExit(final DropTargetEvent event) {
    }

    /**
     * is invoked when a drag operation is going on
     */
    public void dragOver(final DropTargetDragEvent event) {
        dndEventLocation = viewToModel(event.getLocation());
        try {
            setCaretPosition(dndEventLocation);
        } catch (Exception e) {
            Utils.processException(e);
        }
    }

    public void drop(final DropTargetDropEvent event) {
        dndEventLocation = viewToModel(event.getLocation());
        if ((dndEventLocation >= lastSelStart) &&
                (dndEventLocation <= lastSelEnd)) {
            event.rejectDrop();
            select(lastSelStart, lastSelEnd);
        } else {
            try {
                final Transferable transferable = event.getTransferable();
                if (transferable.isDataFlavorSupported(df)) {
                    final StyledContent s = (StyledContent) transferable.getTransferData(df);
                    doDrop(event, s);
                } else if (transferable.isDataFlavorSupported(DataFlavor.stringFlavor)) {
                    final String s = (String) transferable.getTransferData(DataFlavor.stringFlavor);
                    doDrop(event, s);
                } else {
                    event.rejectDrop();
                }
            } catch (Exception e) {
                Utils.processException(e);
                event.rejectDrop();
            }
        }
    }

    /**
     * do the drop operation consisting of adding the dragged element and
     * necessarily removing the dragged element from the original position
     */
    private void doDrop(final DropTargetDropEvent event, final Object s) {
        try {
            this.firePropertyChange("undoredo", false, true);
            int removeOffset = 0;
            int moveOffset = 0;
            final int newSelStart;
            final int newSelEnd;
            event.acceptDrop(DnDConstants.ACTION_MOVE);
            addElement(s);
            if (dndEventLocation < lastSelStart) {
                removeOffset = s.toString().length();
            } else {
                moveOffset = s.toString().length();
            }
            newSelEnd = dndEventLocation + (lastSelEnd - lastSelStart) - moveOffset;
            newSelStart = dndEventLocation - moveOffset;
            if (dragStartedHere) {
                lastSelStart += removeOffset;
                lastSelEnd += removeOffset;
                removeElement();
            }
            lastSelEnd = newSelEnd;
            lastSelStart = newSelStart;
            select(lastSelStart, lastSelEnd);
            event.getDropTargetContext().dropComplete(true);
        } finally {
            this.firePropertyChange("undoredo", true, false);
        }

    }

    /**
     * is invoked if the user modifies the current drop gesture
     */
    public void dropActionChanged(final DropTargetDragEvent event) {
    }

    public void addElement(final Object s) {
        try {
            setCaretPosition(dndEventLocation);
            if (s instanceof StyledContent) {
                replaceSelection((StyledContent) s);
            } else if (s instanceof String) {
                replaceSelection((String) s);
            }
        } catch (Exception e) {
            Utils.processException(e);
        }
    }

    public void removeElement() {
        select(lastSelStart, lastSelEnd);
        replaceSelection("");
    }

    public void finalize() {
        // can help GC to clean up memory
        undoManager.discardAllEdits();
        undoManager = null;
        defaultInputMap = null;
    }

    // ------ end of drag and drop implementation ----------------------------

//    private void markParenthis(final Element paraElement, final int offset) {
//        final MutableAttributeSet set = (MutableAttributeSet) paraElement.getElement(paraElement.getElementIndex(offset)).getAttributes();
//        getDoc().setCharacterAttributes(offset, 1, RiderStyles.updateInvertAttributes(set), false);
//    }


//    public void caretUpdate(final CaretEvent e) {
//        final Element paraElement = getDoc().getParagraphElement(e.getDot());
//        SwingUtilities.invokeLater(new Runnable() {
//            public void run() {
//                if (leftParenthis != -1) {
//                    markParenthis(paraElement, leftParenthis);
//                    markParenthis(paraElement, rightParenthis);
//                    leftParenthis = rightParenthis = -1;
//                }
//                if (RiderStyles.isMath(paraElement)) {
//                    final int startOffset = paraElement.getStartOffset();
//                    final int relPosition = e.getDot() - startOffset;
//                    try {
//                        final String paraText = getDocument().getText(startOffset, paraElement.getEndOffset() - startOffset);
//                        if (paraText.length() > 0) {
//                            if (relPosition < paraText.length() - 1 && paraText.charAt(relPosition) == '(') {
//                                rightParenthis = findRightParenthis(relPosition, paraText.toCharArray());
//                                if (rightParenthis != -1) {
//                                    leftParenthis = e.getDot();
//                                    rightParenthis += startOffset;
//                                }
//                            }
//                            if (leftParenthis != -1) {
//                                markParenthis(paraElement, leftParenthis);
//                                markParenthis(paraElement, rightParenthis);
//                            }
//                        }
//                    } catch (BadLocationException ex) {
//                        Utils.processException(ex);
//                    }
//                }
//            }
//        });
//    }
//
//    private static int findRightParenthis(final int relPosition, final char[] chars) {
//        final int length = chars.length;
//        for (int i = relPosition; i < length; ++i)
//            if (chars[i] == ')')
//                return i;
//        return -1;
//    }
}