/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.area;

import net.wordrider.area.views.RiderViewFactory;
import net.wordrider.ti89.InvalidDataTypeException;
import net.wordrider.ti89.NotSupportedFileException;
import net.wordrider.ti89.TIImageDecoder;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.List;


/**
 * #                          #                    &
 * U : underline              1 : small font       L : Left alignement
 * V : vector                 2 : medium font      R : Right alignement
 * I : invert                 3 : big font         C : Centered
 * N : underline dotted                            \ : Margin 0 pixel
 * S : strikethrought                              , : Margin 10 pixels
 * E : exponent                                    ; : Margin 20 pixels
 * W : Word Wrap                                   . : margin 30 pixels
 *                                                 E : Pretty printed expression
 *                                                 - : line separator
 *                                                 = : double "-"
 */

/**
 * @author Vity
 */
public final class RiderEditorKit extends StyledEditorKit {
    private static final Action[] defaultActions = {new DefaultKeyTypedAction(), new InsertTabAction()};

    public static final char[] TI92PC_UNICODE_FONT = {'\u20AC', '\u0081', '\u201A', '\u0192', '\u201E', '\u2026', '\u2020', '\u2021', '\u02C6', '\u2030', '\u0160', '\u2039', '\u0152', '\u008D', '\u017D', '\u008F', '\u0090', '\u2018', '\u2019', '\u201C', '\u201D', '\u2013', '\u2014', '\u00C6', '\u02DC', '\u2122', '\u0161', '\u203A', '\u0153', '\u009D', '\u017E', '\u0178', '\u00A1', '\u00A2', '\u00A3', '\u00A4', '\u00A5', '\u00A6', '\u007C', '\u00A7', '\u00A7', '\u00A8', '\u00AA', '\u00D7', '\u00A9', '\u00AA', '\u00A8', '\u00AB', '\u00AC', '\u00AD', '\u00AE', '\u00AF', '\u00B0', '\u00B1', '\u00B6', '\u00B2', '\u00B3', '\u00B4', '\u00B5', '\u00DE', '\u00B6', '\u00B7', '\u00B8', '\u00B9', '\u00C0', '\u00C1', '\u00C2', '\u00C3', '\u00C4', '\u00C5', '\u00E6', '\u00C7', '\u00C8', '\u00C9', '\u00CA', '\u00CB', '\u00CD', '\u00CC', '\u00CD', '\u00CF', '\u00D0', '\u00D1', '\u00D2', '\u00D3', '\u00D4', '\u00D5', '\u00D6', '\u0078', '\u00D8', '\u00D9', '\u00DA', '\u00DB', '\u00DC', '\u00DD', '\u00FE', '\u00DF', '\u00E0', '\u00E1', '\u00E2', '\u00E3', '\u00E4', '\u00E5', '\u00E6', '\u00E7', '\u00E8', '\u00E9', '\u00EA', '\u00EB', '\u00EC', '\u00ED', '\u00EE', '\u00EF', '\u00F0', '\u00F1', '\u00F2', '\u00F3', '\u00F4', '\u00F5', '\u00F6', '\u00F7', '\u00F8', '\u00F9', '\u00FA', '\u00FB', '\u00FC', '\u00FD', '\u00FE', '\u00FF'};

    public static final HashMap TI2UNICODE;

    static {
        TI2UNICODE = new HashMap(6);
        TI2UNICODE.put(new Character((char) 18), new Character('\u00D0'));
        TI2UNICODE.put(new Character((char) 22), new Character('\u00BB'));
        TI2UNICODE.put(new Character((char) 28), new Character('\u00BC'));
        TI2UNICODE.put(new Character((char) 29), new Character('\u00BD'));
        TI2UNICODE.put(new Character((char) 30), new Character('\u00BE'));
        TI2UNICODE.put(new Character((char) 31), new Character('\u00BF'));
    }

    public final static BitSet WORD_SEPARATORS;

    static {
        WORD_SEPARATORS = new BitSet(256);
        WORD_SEPARATORS.set(' ');
        WORD_SEPARATORS.set('\n');
        WORD_SEPARATORS.set('\r');
        WORD_SEPARATORS.set('\f');
        WORD_SEPARATORS.set('.');
        WORD_SEPARATORS.set(',');
        WORD_SEPARATORS.set(':');
        WORD_SEPARATORS.set('(');
        WORD_SEPARATORS.set(')');
        WORD_SEPARATORS.set('[');
        WORD_SEPARATORS.set(']');
        WORD_SEPARATORS.set('{');
        WORD_SEPARATORS.set('}');
        WORD_SEPARATORS.set('<');
        WORD_SEPARATORS.set('>');
        WORD_SEPARATORS.set('\\');
        WORD_SEPARATORS.set('@');
        WORD_SEPARATORS.set('?');
        WORD_SEPARATORS.set('!');
        WORD_SEPARATORS.set(';');
        WORD_SEPARATORS.set('*');
        WORD_SEPARATORS.set('+');
        WORD_SEPARATORS.set('-');
        WORD_SEPARATORS.set('\"');
        WORD_SEPARATORS.set('\'');
        WORD_SEPARATORS.set('$');
        WORD_SEPARATORS.set('=');
        WORD_SEPARATORS.set('&');
    }


    public final ViewFactory getViewFactory() {
        return new RiderViewFactory();
    }

    public RiderEditorKit() {
        super();    //call to super
    }

    public Action[] getActions() {
        return TextAction.augmentList(super.getActions(), defaultActions);
    }

    private final void initDefaultStyles() {
        getInputAttributes().addAttributes(RiderStyles.normalStyle);
    }


    private static final char getUnicodeFromTIChar(final char ch) {
        if (ch >= 128 && ch <= 0xFF)
            return TI92PC_UNICODE_FONT[ch - 128];
        if (ch < 32) {      //speed optimization
            final Character tmp = (Character) TI2UNICODE.get(new Character(ch));
            return (tmp != null) ? tmp.charValue() : ch;
        } else
            return ch;
    }

    public void setTextRiderText(final JTextComponent area, final String text, final String dir) throws BadLocationException {
        final RiderDocument doc = (RiderDocument) createDefaultDocument();
        initDefaultStyles();
        char ch;
        boolean startLine = true, bookmark = true;
        MutableAttributeSet temp;
        int offset = 0;
        StringBuffer buffer = new StringBuffer();
        final List bookmarkList = new LinkedList(), mathList = new LinkedList();
        final int length = text.length();
        final char[] textArray = text.toCharArray();
        for (int i = 0; i < length; ++i) {
            ch = textArray[i];
            switch (ch) {
                case ' ':
                    if (startLine)
                        bookmark = false;
                    else
                        buffer.append(' ');
                    break;
                case '\r':
                    buffer.append('\n');
                    startLine = bookmark = true;
                    break;
                case '#':
                    if (i + 1 != length) {
                        switch (textArray[++i]) {
                            case '1':
                                temp = RiderStyles.miniStyle;
                                break;
                            case '2':
                                temp = RiderStyles.normalStyle;
                                break;
                            case '3':
                                temp = RiderStyles.maxiStyle;
                                break;
                            case 'U':
                                temp = RiderStyles.updateUnderlineAttributes(getInputAttributes());
                                break;
                            case 'V':
                                temp = RiderStyles.updateVectorAttributes(getInputAttributes());
                                break;
                            case 'I':
                                temp = RiderStyles.updateInvertAttributes(getInputAttributes());
                                break;
                            case 'N':
                                temp = RiderStyles.updateUnderlineDottedAttributes(getInputAttributes());
                                break;
                            case 'S':
                                temp = RiderStyles.updateStrikedAttributes(getInputAttributes());
                                break;
                            case 'E':
                                temp = RiderStyles.updateExposantAttributes(getInputAttributes());
                                break;
                            case 'W':
                                temp = RiderStyles.updateWordWrapAttributes(getInputAttributes());
                                break;
                            default :
                                --i;
                                temp = null;
                                buffer.append('#');
                                break;
                        }
                        if (temp != null) {
                            if (buffer.length() != 0) {
                                doc.insertString(offset, buffer.toString(), getInputAttributes());
                                offset += buffer.length();
                                buffer = new StringBuffer();
                            }
                            getInputAttributes().addAttributes(temp);
                        }
                    } else
                        buffer.append('#');
                    break;
                case '&':
                    if (startLine && !bookmark && i + 1 != length) {
                        boolean keyword = true;
                        int component = -1;
                        Image image = null;
                        temp = null;
                        switch (text.charAt(++i)) {
                            case '-':
                                component = RiderStyles.SINGLE_LINE;
                                break;
                            case '=':
                                component = RiderStyles.DOUBLE_LINE;
                                break;
                            case 'L':
                            case '\\':
                                temp = RiderStyles.aligmentLeftStyle;
                                break;
                            case 'R':
                                temp = RiderStyles.aligmentRightStyle;
                                break;
                            case 'C':
                                temp = RiderStyles.aligmentCenteredStyle;
                                break;
                            case ',':
                                temp = RiderStyles.margin10Style;
                                break;
                            case ';':
                                temp = RiderStyles.margin20Style;
                                break;
                            case '.':
                                temp = RiderStyles.margin30Style;
                                break;
                            case 'E':
                                // temp = RiderStyles.mathStyle;
                                mathList.add(new Integer(doc.getLength() + buffer.length()));
                                break;
                            case 'P': //picture
                                final String fileName = getPictureFileName(textArray, i + 1);
                                if (fileName != null) {
                                    i += fileName.length();
                                    final File f = getPictureFile(dir, fileName);
                                    if (f != null)
                                        image = loadPicture(f, dir, fileName);
                                } else
                                    Utils.log(Utils.WARN_LOG, "Picture filename couldn't be extracted");
                                break;
                            default :
                                --i;
                                //buffer.append(getUnicodeFromTIChar(ch));
                                keyword = false;
                                break;
                        }
                        if (keyword) {
                            if (buffer.length() != 0) {
                                doc.insertString(offset, buffer.toString(), getInputAttributes());
                                offset += buffer.length();
                                buffer = new StringBuffer();
                            }
                            if (temp != null)
                                doc.setParagraphAttributes(offset, 0, temp, true);
                            else {
                                if (component != -1) {
                                    doc.insertSeparateLine(area, offset, component);
                                    ++offset;
                                } else if (image != null) {
                                    doc.insertPicture(offset, image);
                                    ++offset;
                                }
                            }
                        }
                    } else
                        buffer.append('&');
                    startLine = false;
                    break;
                case '\f':
                    if (startLine) {
                        bookmarkList.add(new Integer(doc.getLength() + buffer.length()));
                        bookmark = false;
                        break;
                    }        //else default
                default:
                    buffer.append(getUnicodeFromTIChar(ch));
                    startLine = false;
                    break;
            }
        }
        if (buffer.length() > 0)
            doc.insertString(offset, buffer.toString(), getInputAttributes());
        //int paraEl;
        //RiderStyles.setProperty(new SimpleAttributeSet(),RiderStyles.STYLE_WORDWRAP, false);
        for (final Iterator iterator = bookmarkList.iterator(); iterator.hasNext();)
            doc.putBookmark(((Integer) iterator.next()).intValue());

        for (final Iterator iterator = mathList.iterator(); iterator.hasNext();) {
            doc.activatePrettyPrint(((Integer) iterator.next()).intValue());
            //   paraEl = ((Integer) iterator.next()).intValue();
            //paraElement = doc.getParagraphElement(paraEl);
            //doc.setParagraphAttributes(paraEl, 0, RiderStyles.mathStyle, true);
            //doc.setCharacterAttributes(paraEl, paraEl, );
            //doc.setLogicalStyle(((Integer) iterator.next()).intValue(), RiderStyles.mathStyle);
        }
        area.setDocument(doc);

    }

    private static final String getPictureFileName(final char[] textArray, final int offset) {
        final int arrayLength = textArray.length;
        int i = offset;
        for (; i < arrayLength && textArray[i] != '\r'; ++i) ; // to the end of file or new line
        return (i != offset) ? new String(textArray, offset, i - offset) : null;
    }

    private static final File getPictureFile(String dir, String fileName) {
        if (!dir.endsWith(File.separator))
            dir += File.separator;
        File f = new File(dir + (!fileName.endsWith(".89i") ? fileName + ".89i" : fileName));
        if (!(f.exists() && f.isFile())) {
            //Utils.log(Utils.WARN_LOG, "Cannot load the image file " + f.getAbsolutePath());
            f = new File(dir + fileName);
            if (!(f.exists() && f.isFile())) {
                //Utils.log(Utils.WARN_LOG, "Automatic attempt for loading image file failed " + f.getAbsolutePath());
                final int index = fileName.lastIndexOf(File.separatorChar);
                if (index != -1) {
                    fileName = fileName.substring(index);
                    f = new File(dir + fileName);
                    if (!(f.exists() && f.isFile())) {
                        //Utils.log(Utils.WARN_LOG, "Automatic attempt for loading image file failed " + f.getAbsolutePath());
                        f = new File(dir + (!fileName.endsWith(".89i") ? fileName + ".89i" : fileName));
                        if (!(f.exists() && f.isFile())) {
                            Utils.log(Utils.WARN_LOG, "Automatic attempt for loading image file failed " + f.getAbsolutePath());
                            return null;
                        }
                    }
                } else {
                    Utils.log(Utils.WARN_LOG, "Automatic attempt for loading image file failed " + f.getAbsolutePath());
                    return null;
                }
            }
        }
        return f;
    }

    private static final Image loadPicture(final File f, final String dir, final String fileName) {
        final TIImageDecoder ti = new TIImageDecoder();
        try {
            if (!ti.openFromFile(f))
                Utils.log(Utils.WARN_LOG, "Image was loaded but the file might be corrupted (invalid checksum)");
            ti.setOriginalFileName(fileName);
            ti.setOriginalFileDir(dir);
            ti.setImagePath(f.getParent());
            return Toolkit.getDefaultToolkit().createImage(ti);
        } catch (InvalidDataTypeException e) {
            Utils.log(Utils.WARN_LOG, e.getMessage());
            return null;
        } catch (NotSupportedFileException e) {
            Utils.log(Utils.WARN_LOG, e.getMessage());
            return null;
        } catch (IOException e) {
            Utils.log(Utils.WARN_LOG, "Cannot load image " + e.getMessage());
            return null;
        } catch (Exception e) {
            Utils.log(Utils.ERROR_LOG, "Fatal error exception " + e.getMessage());
            return null;
        }

    }

    public final Document createDefaultDocument() {
        return new RiderDocument();
    }

    private static final class InsertTabAction extends DefaultEditorKit.InsertTabAction {
        public void actionPerformed(final ActionEvent e) {
            final JTextComponent target = getTextComponent(e);
            if (target != null) {
                if ((!target.isEditable()) || (!target.isEnabled())) {
                    UIManager.getLookAndFeel().provideErrorFeedback(target);
                    return;
                }
                target.replaceSelection("  ");
            }
        }

    }

}
