/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.area;

import net.wordrider.ti89.TIImageDecoder;
import net.wordrider.utilities.Consts;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.image.ImageProducer;
import java.io.IOException;
import java.io.InputStream;

/**
 * @author Vity
 */
public final class RiderStyles {
    public static final String TI89MAXLENGTHSTRING = "01234567890123456789012345";
    public static final String TI92MAXLENGTHSTRING = "012345678901234567890123456789012345";
    public final static Stroke DOTTED_STROKE = new BasicStroke(1f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 1f, new float[]{2f}, 0f);

    private static final Object STYLE_INVERTED = "INVERT";
    public static final Object STYLE_UNDERLINEDOTTED = "UNDERLINEDOTTED";
    private static final Object STYLE_BOOKMARK = "BOOKMARK";
    private static final Object STYLE_VECTOR = "VECTOR";
    private static final Object STYLE_MATH = "MATH";
    static final Object STYLE_WORDWRAP = "WORDWRAP";
    public static final String STYLE_ALIGMENT_LEFT = "aligmentLeft";
    public static final String STYLE_ALIGMENT_RIGHT = "aligmentRight";
    public static final String STYLE_ALIGMENT_CENTER = "aligmentCenter";
    public static final String STYLE_FONT_MINI = "miniSize";
    public static final String STYLE_FONT_NORMAL = "normalSize";
    public static final String STYLE_FONT_MAXI = "maxiSize";
    public static final String STYLE_MARGIN10 = "margin10";
    public static final String STYLE_MARGIN20 = "margin20";
    public static final String STYLE_MARGIN30 = "margin30";


    public static final int ALIGN_LEFT = StyleConstants.ALIGN_LEFT;
    public static final int ALIGN_RIGHT = StyleConstants.ALIGN_RIGHT;
    public static final int ALIGN_CENTER = StyleConstants.ALIGN_CENTER;
    public static final int SIZE_MINI = 11;
    public static final int SIZE_NORMAL = 13;
    public static final int SIZE_MAXI = 16;
    public static final int MARGIN_0 = 0;
    public static final int MARGIN_10 = 10;
    public static final int MARGIN_20 = 20;
    public static final int MARGIN_30 = 30;

    public static final int SINGLE_LINE = SeparatorLine.SINGLE_LINE;
    public static final int DOUBLE_LINE = SeparatorLine.DOUBLE_LINE;

    public static final String FONT_NAME = "Ti-92p.ttf";
    public static final String FONT_FAMILY = "Ti92Pluspc";

    private static final Font font = readDefaultFont();

    private static final Font maxiFont = font.deriveFont(Font.BOLD, 15F);
    public static Style normalStyle;
    public static Style miniStyle;
    public static Style maxiStyle;
    private static Style superscriptStyle;
    public static Style margin10Style;
    public static Style margin20Style;
    public static Style margin30Style;
    public static Style aligmentLeftStyle;
    public static Style aligmentRightStyle;
    public static Style aligmentCenteredStyle;
    public static Style underlineStyle;
    private static Style strikethroughStyle;
    public static Style underlineDottedStyle;
    private static Style basicStyle;
    public static Style invertStyle;
    public static Style mathStyle;
    private static Style vectorStyle;
    private static Style wordwrapStyle;
    public static SimpleAttributeSet bookmarkStyle;


    private static Font readDefaultFont() {
        //check for fontfamily in system
        Font result;
        if ((result = new Font(FONT_FAMILY, Font.PLAIN, SIZE_NORMAL)).getFamily().equalsIgnoreCase(FONT_FAMILY))
            return result;
        InputStream stream = null;
        try {
            stream = (RiderStyles.class.getClassLoader()).getResourceAsStream(Consts.FONTDIR + FONT_NAME);
            if (stream != null) {
                result = Font.createFont(Font.TRUETYPE_FONT, stream).deriveFont(Font.PLAIN, SIZE_NORMAL);
            }
        } catch (FontFormatException e) {
            Utils.processException(e);
        } catch (IOException e) {
            Utils.processException(e);
        } finally {
            try {
                if (stream != null)
                    stream.close();
            } catch (IOException e) {
                Utils.processException(e);
            }
        }
        return result;
    }

    public final static boolean isCorrectFont() {
        return font.getFamily().equalsIgnoreCase(FONT_FAMILY);
    }

    public static final void createDefaultStyles(final StyleContext sc) {
        final Style defaultStyle = sc.getStyle(StyleContext.DEFAULT_STYLE);
        StyleConstants.setFontFamily(defaultStyle, font.getFamily());
        StyleConstants.setFontSize(defaultStyle, SIZE_NORMAL);
        StyleConstants.setForeground(defaultStyle, getAreaFontForeground());
        StyleConstants.setBackground(defaultStyle, getAreaFontBackground());

        basicStyle = sc.addStyle("basic", null);
        StyleConstants.setFontFamily(basicStyle, font.getFamily());
        //StyleConstants.setFontSize(basicStyle, SIZE_NORMAL);
        StyleConstants.setForeground(basicStyle, getAreaFontForeground());
        StyleConstants.setBackground(basicStyle, getAreaFontBackground());
        //StyleConstants.setLeftIndent(basicStyle,0);
        //StyleConstants.setAlignment(basicStyle,StyleConstants.ALIGN_LEFT);
        normalStyle = sc.addStyle(STYLE_FONT_NORMAL, basicStyle);
        StyleConstants.setFontSize(normalStyle, SIZE_NORMAL);
        StyleConstants.setBold(normalStyle, false);
        //StyleConstants.setSuperscript(normalStyle, false);
        miniStyle = sc.addStyle(STYLE_FONT_MINI, basicStyle);
        StyleConstants.setFontSize(miniStyle, SIZE_MINI);
        //     StyleConstants.setSuperscript(miniStyle, false);
        StyleConstants.setBold(miniStyle, false);
        maxiStyle = sc.addStyle(STYLE_FONT_MAXI, basicStyle);
        StyleConstants.setFontSize(maxiStyle, SIZE_MAXI);
        StyleConstants.setBold(maxiStyle, true);
        StyleConstants.setSuperscript(maxiStyle, false);

        superscriptStyle = sc.addStyle("superscript", miniStyle);
        StyleConstants.setSuperscript(superscriptStyle, true);
        //superscriptStyle.addAttributes(normalStyle);

        aligmentLeftStyle = sc.addStyle(STYLE_ALIGMENT_LEFT, basicStyle);
        StyleConstants.setLeftIndent(aligmentLeftStyle, 0);
        //setProperty(aligmentLeftStyle,STYLE_MATH, false);
        StyleConstants.setAlignment(aligmentLeftStyle, StyleConstants.ALIGN_LEFT);
        aligmentRightStyle = sc.addStyle(STYLE_ALIGMENT_RIGHT, aligmentLeftStyle);
        StyleConstants.setAlignment(aligmentRightStyle, StyleConstants.ALIGN_RIGHT);
        StyleConstants.setLeftIndent(aligmentRightStyle, 0);
        aligmentCenteredStyle = sc.addStyle(STYLE_ALIGMENT_CENTER, aligmentLeftStyle);
        StyleConstants.setAlignment(aligmentCenteredStyle, StyleConstants.ALIGN_CENTER);
        StyleConstants.setLeftIndent(aligmentCenteredStyle, 0);

        margin10Style = sc.addStyle(STYLE_MARGIN10, aligmentLeftStyle);
        StyleConstants.setLeftIndent(margin10Style, 10);
        StyleConstants.setAlignment(margin10Style, StyleConstants.ALIGN_LEFT);
        margin20Style = sc.addStyle(STYLE_MARGIN20, aligmentLeftStyle);
        StyleConstants.setLeftIndent(margin20Style, 20);
        StyleConstants.setAlignment(margin20Style, StyleConstants.ALIGN_LEFT);
        margin30Style = sc.addStyle(STYLE_MARGIN30, aligmentLeftStyle);
        StyleConstants.setLeftIndent(margin30Style, 30);
        StyleConstants.setAlignment(margin30Style, StyleConstants.ALIGN_LEFT);

        underlineStyle = sc.addStyle("underline", aligmentLeftStyle); //underline cannot to rewrite underline-dotted style on false
        //setProperty(underlineStyle, STYLE_UNDERLINEDOTTED, false);

        StyleConstants.setUnderline(underlineStyle, true);
        strikethroughStyle = sc.addStyle("strikethrough", basicStyle);
        StyleConstants.setStrikeThrough(strikethroughStyle, true);


        underlineDottedStyle = sc.addStyle("underlineDotted", basicStyle);
        //!dont delete! problems StyleConstants.setBackground(underlineDottedStyle, Color.WHITE); problems
        StyleConstants.setUnderline(underlineDottedStyle, false);
        setProperty(underlineDottedStyle, STYLE_UNDERLINEDOTTED);


        invertStyle = sc.addStyle("invert", basicStyle);
        StyleConstants.setBackground(invertStyle, getAreaFontForeground());
        StyleConstants.setForeground(invertStyle, getAreaFontBackground());
        setProperty(invertStyle, STYLE_INVERTED);

        mathStyle = sc.addStyle("math", normalStyle);
        setProperty(mathStyle, STYLE_MATH);
        mathStyle.addAttributes(aligmentLeftStyle);
        //StyleConstants.setLeftIndent(mathStyle, 0);
        //setProperty(aligmentLeftStyle,STYLE_MATH, false);
        //StyleConstants.setAlignment(mathStyle, StyleConstants.ALIGN_LEFT);
        StyleConstants.setBackground(mathStyle, getAreaFontBackground());
        //mathStyle = (Style) setProperty(mathStyle, STYLE_WORDWRAP, false);


        vectorStyle = sc.addStyle("vector", basicStyle);
        setProperty(vectorStyle, STYLE_VECTOR);

        wordwrapStyle = sc.addStyle("wordwrap", wordwrapStyle);
        setProperty(wordwrapStyle, STYLE_WORDWRAP);

        bookmarkStyle = new SimpleAttributeSet();
        //bookmarkStyle.addAttribute(STYLE_BOOKMARK,new Boolean(true));
    }

    public static MutableAttributeSet setProperty(final MutableAttributeSet attributes, final Object style, final boolean value) {
        attributes.addAttribute(style, Boolean.valueOf(value));
        return attributes;
    }

    private static void setProperty(final MutableAttributeSet attributes, final Object style) {
        attributes.addAttribute(style, Boolean.valueOf(true));
    }


    public static final boolean isReadonlySection(final Element paraElement) {
        final int count = paraElement.getElementCount();
        String elName;
        for (int j = 0; j < count; ++j) {
            elName = paraElement.getElement(j).getName();
            if (elName.equals(StyleConstants.ComponentElementName) || elName.equals(StyleConstants.IconElementName))
                return true;

        }
        return false;
    }

    public static final Component isMoveableComponent(final Element paraElement) {
        final int count = paraElement.getElementCount();
        String elName;
        Element childElement;
        for (int j = 0; j < count; ++j) {
            childElement = paraElement.getElement(j);
            elName = childElement.getName();
            if (elName.equals(StyleConstants.ComponentElementName) )
                return StyleConstants.getComponent(childElement.getAttributes());

        }
        return null;
    }



    public static final boolean isUnderLine(final AttributeSet attributes) {
        return StyleConstants.isUnderline(attributes);
    }

    public static final boolean isUnderLineDotted(final AttributeSet attributes) {
        final Object o = attributes.getAttribute(STYLE_UNDERLINEDOTTED);
        return o != null && ((Boolean) o).booleanValue();
    }

    public static final boolean isExposant(final AttributeSet attributes) {
        return StyleConstants.isSuperscript(attributes);
    }

    public static final boolean isInvert(final AttributeSet attributes) {
        final Object o = attributes.getAttribute(STYLE_INVERTED);
        return o != null && ((Boolean) o).booleanValue();
    }

    public static final boolean isVector(final AttributeSet attributes) {
        final Object o = attributes.getAttribute(STYLE_VECTOR);
        return o != null && ((Boolean) o).booleanValue();
    }


    public static final boolean isSize(final AttributeSet attributes, final int size) {
        return StyleConstants.getFontSize(attributes) == size;
    }

    public static final boolean isMath(final Element el) {
        final Object obj = el.getAttributes().getAttribute(STYLE_MATH);
        return obj != null && ((Boolean) obj).booleanValue();
    }

    public static final boolean isStrikeOut(final AttributeSet attributes) {
        return StyleConstants.isStrikeThrough(attributes);
    }

    public static final boolean isAligment(final AttributeSet attributes, final int aligment) {
        return StyleConstants.getAlignment(attributes) == aligment;
    }

    public static final boolean isMargin(final AttributeSet attributes, final int indent) {
        return (int) StyleConstants.getLeftIndent(attributes) == indent;
    }

    public static final boolean isWordWrap(final AttributeSet attributes) {
        final Object o = attributes.getAttribute(STYLE_WORDWRAP);
        return o != null && ((Boolean) o).booleanValue();
    }


    public static final boolean isSupportedComponent(final Element el) {
        if (el.getName().equals(StyleConstants.ComponentElementName)) {
            return (el.getAttributes().getAttribute(StyleConstants.ComponentAttribute) instanceof SeparatorLine);
        }
        return false;
    }

    public static final boolean isComponentLine(final Element el, final int lineType) {
        if (el.getName().equals(StyleConstants.ComponentElementName)) {
            final Object obj = el.getAttributes().getAttribute(StyleConstants.ComponentAttribute);
            if (obj instanceof SeparatorLine)
                if (((SeparatorLine) obj).getLineType() == lineType)
                    return true;
        }
        return false;
    }

    public static final TIImageDecoder isSupportedImage(final Element el) {
        if (el.getName().equals(StyleConstants.IconElementName)) {
            final ImageIcon icon = (ImageIcon) el.getAttributes().getAttribute(StyleConstants.IconAttribute);
            final ImageProducer imageProducer = icon.getImage().getSource();
            if (imageProducer instanceof TIImageDecoder)
                return (TIImageDecoder) imageProducer;
        }
        return null;
    }
    

    public static final ImageIcon isImage(final Element el) {
        if (el.getName().equals(StyleConstants.IconElementName)) {
            final ImageIcon icon = (ImageIcon) el.getAttributes().getAttribute(StyleConstants.IconAttribute);
            final ImageProducer imageProducer = icon.getImage().getSource();
            if (imageProducer instanceof TIImageDecoder)
                return icon;
        }
        return null;
    }


    public static final boolean isBookmark(final Element el) {
        final Object obj = el.getAttributes().getAttribute(STYLE_BOOKMARK);
        return obj != null && obj.equals(el);
    }

//    public static final boolean isBookmark(final AttributeSet attributes) {
//        final Object o = attributes.getAttribute(STYLE_BOOKMARK);
//        return o != null && ((Boolean)o).booleanValue();
//    }


    public static final int getAligment(final AttributeSet attributes) {
        return StyleConstants.getAlignment(attributes);
    }

    public static final int getMargin(final AttributeSet attributes) {
        return (int) StyleConstants.getLeftIndent(attributes);
    }

    public static final int getFontSize(final AttributeSet attributes) {
        return StyleConstants.getFontSize(attributes);
    }


    public static final Font getAreaFont() {
        return font;
    }

    public static final Font getAreaBigFont() {
        return maxiFont;
    }

    private static Color getAreaFontForeground() {
        return Color.BLACK;
    }

    private static Color getAreaFontBackground() {
        return Color.WHITE;
    }

    public static final MutableAttributeSet updateVectorAttributes(final MutableAttributeSet inputAttributeSet) {
        return (isVector(inputAttributeSet)) ? setProperty(new SimpleAttributeSet(), STYLE_VECTOR, false) : vectorStyle;
    }

    public static final MutableAttributeSet updateUnderlineAttributes(final MutableAttributeSet inputAttributeSet) {
        return (isUnderLine(inputAttributeSet)) ? setProperty(new SimpleAttributeSet(), StyleConstants.Underline, false) : underlineStyle;
    }

    public static final MutableAttributeSet flipUnderlineAttributes(final MutableAttributeSet inputAttributeSet) {
        final MutableAttributeSet set = (isUnderLine(inputAttributeSet)) ? setProperty(new SimpleAttributeSet(), StyleConstants.Underline, false) : new SimpleAttributeSet(underlineStyle.copyAttributes());
        return setProperty(set, STYLE_UNDERLINEDOTTED, false);
    }

    public static final MutableAttributeSet flipUnderlineDottedAttributes(final MutableAttributeSet inputAttributeSet) {
        final MutableAttributeSet set = (isUnderLineDotted(inputAttributeSet)) ? setProperty(new SimpleAttributeSet(), STYLE_UNDERLINEDOTTED, false) : new SimpleAttributeSet(underlineDottedStyle.copyAttributes());
        StyleConstants.setUnderline(set, false);
        return set;
    }

    public static final MutableAttributeSet updateUnderlineDottedAttributes(final MutableAttributeSet inputAttributeSet) {
        return (isUnderLineDotted(inputAttributeSet)) ? setProperty(new SimpleAttributeSet(), STYLE_UNDERLINEDOTTED, false) : underlineDottedStyle;
    }

    public static final MutableAttributeSet updateWordWrapAttributes(final MutableAttributeSet inputAttributeSet) {
        return (isWordWrap(inputAttributeSet)) ? setProperty(new SimpleAttributeSet(), STYLE_WORDWRAP, false) : wordwrapStyle;
    }

    public static final MutableAttributeSet updateInvertAttributes(final MutableAttributeSet inputAttributeSet) {
        return (isInvert(inputAttributeSet)) ? setProperty(new SimpleAttributeSet(basicStyle), STYLE_INVERTED, false) : invertStyle;
    }

    public static final MutableAttributeSet updateStrikedAttributes(final MutableAttributeSet inputAttributeSet) {
        return (isStrikeOut(inputAttributeSet)) ? setProperty(new SimpleAttributeSet(), StyleConstants.StrikeThrough, false) : strikethroughStyle;
    }

    public static final MutableAttributeSet updateExposantAttributes(final MutableAttributeSet inputAttributeSet) {
        return (isExposant(inputAttributeSet)) ? setProperty(new SimpleAttributeSet(), StyleConstants.Superscript, false) : superscriptStyle;
    }

    public static final MutableAttributeSet updateBookmark(final Element el) {
        bookmarkStyle.addAttribute(STYLE_BOOKMARK, el);
        return bookmarkStyle;
    }

    public static Color getMathStyleColor() {
        return Color.BLUE;
    }

}
