/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.area;


import net.wordrider.ti89.TIImageDecoder;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * @author Vity
 */
public final class StyledContent {

    private final List clipContent = new Vector(10);

    private static final class SelectedElementsIterator {
        public static final int REVERSE_DIRECTION = 0;
        public static final int FORWARD_DIRECTION = 1;
        private final Element root;
        private Element activeElement;
        private int activeParaIndex, activeLeafIndex, startParaIndex, startLeafIndex;
        private final int endParaIndex;
        private final int endLeafIndex;
        private int activeParaElementsCount;
        private boolean isFirst;


        public SelectedElementsIterator(final Element docRoot, final int startOffset, final int endOffset, final int direction) {
            this.root = docRoot;
            if (direction == FORWARD_DIRECTION) {
                this.activeElement = docRoot.getElement(startParaIndex = activeParaIndex = docRoot.getElementIndex(startOffset));
                this.startLeafIndex = this.activeLeafIndex = this.activeElement.getElementIndex(startOffset);
                this.activeParaElementsCount = this.activeElement.getElementCount();
                this.endParaIndex = docRoot.getElementIndex(endOffset);
                this.endLeafIndex = docRoot.getElement(this.endParaIndex).getElementIndex(endOffset);
            } else {
                this.activeElement = docRoot.getElement(activeParaIndex = docRoot.getElementIndex(endOffset));
                this.activeLeafIndex = this.activeElement.getElementIndex(endOffset);
                this.activeParaElementsCount = this.activeElement.getElementCount();
                this.endParaIndex = docRoot.getElementIndex(startOffset);
                this.endLeafIndex = docRoot.getElement(this.endParaIndex).getElementIndex(startOffset);
            }
            this.isFirst = true;
        }

        public final Element next() {
            if (activeParaIndex < endParaIndex) {
                if (activeLeafIndex < activeParaElementsCount) {
                    return this.activeElement.getElement(activeLeafIndex++);
                } else {
                    this.activeElement = root.getElement(++activeParaIndex);
                    activeParaElementsCount = this.activeElement.getElementCount();
                    activeLeafIndex = 1;
                    return this.activeElement.getElement(0);
                }
            } else
                return (activeLeafIndex <= endLeafIndex && activeParaIndex == endParaIndex) ? this.activeElement.getElement(activeLeafIndex++) : null;
        }

        public final boolean isLastLeafElement() {
            return (activeParaIndex == endParaIndex && activeLeafIndex - 1 == endLeafIndex);
        }

        public final boolean isFirstLeafElement() {
            return (activeParaIndex == startParaIndex && activeLeafIndex - 1 == startLeafIndex);
        }


        public final Element previous() {
            if (activeParaIndex > endParaIndex) {
                if (activeLeafIndex >= 0) {
                    return this.activeElement.getElement(activeLeafIndex--);
                } else {
                    this.activeElement = root.getElement(--activeParaIndex);
                    activeParaElementsCount = this.activeElement.getElementCount();
                    activeLeafIndex = activeParaElementsCount - 2;
                    return this.activeElement.getElement(activeParaElementsCount - 1);
                }
            } else
                return (activeLeafIndex >= endLeafIndex && activeParaIndex == endParaIndex) ? this.activeElement.getElement(activeLeafIndex--) : null;
        }

        public final Element nextElement() {
            if (activeParaIndex < endParaIndex) {
                if (isFirst) {
                    isFirst = false;
                    return this.activeElement;
                }
                if (activeLeafIndex < activeParaElementsCount) {
                    return this.activeElement.getElement(activeLeafIndex++);
                } else {
                    this.activeElement = root.getElement(++activeParaIndex);
                    activeParaElementsCount = this.activeElement.getElementCount();
                    activeLeafIndex = 0;
                    return this.activeElement;
                }
            } else
                return (activeLeafIndex <= endLeafIndex && activeParaIndex == endParaIndex) ? this.activeElement.getElement(activeLeafIndex++) : null;
        }


    }

    private static final class StyledCopyContent {
        public static final int CONTENT_TEXT = 0;
        public static final int CONTENT_SEPARATOR = 1;
        public static final int CONTENT_IMAGE = 2;

        private final Object content;
        private AttributeSet attr;
        private final int contentType;


        public final Object getContent() {
            return content;
        }

        public final AttributeSet getAttr() {
            return attr;
        }

        public final int getContentType() {
            return contentType;
        }

        public StyledCopyContent(final String text, final AttributeSet attr) {
            this.content = text;
            this.attr = attr;
            this.contentType = CONTENT_TEXT;
        }

        public StyledCopyContent(final ImageIcon icon) {
            this.content = Toolkit.getDefaultToolkit().createImage(icon.getImage().getSource());
            this.contentType = CONTENT_IMAGE;
        }

        public StyledCopyContent(final int separatorLineType) {
            this.content = new Integer(separatorLineType);
            this.contentType = CONTENT_SEPARATOR;
        }

        public final String toString() {
            switch (contentType) {
                case CONTENT_TEXT:
                    return content.toString();
                case CONTENT_SEPARATOR:
                    return SeparatorLine.getStringRepresentation(((Integer) content).intValue());
                case CONTENT_IMAGE:
                    {
                        final TIImageDecoder imageProducer = (TIImageDecoder) ((Image) content).getSource();
                        return imageProducer.getStringRepresentation();
                    }
                default :
                    return content.toString();
            }

        }
    }

    /**
     * construct a new <code>StyledContent</code> object and load it with
     * an already selected portion of text from a
     * <code>StyledEditorPane</code>
     *
     * @param src - StyledEditorPane that has selected the text portion to be taken
     */
    public StyledContent(final RiderArea src) throws BadLocationException {
        final Document doc = src.getDocument();
        final int selStart = src.getSelectionStart();
        final int selEnd = src.getSelectionEnd();
        final int max = Math.max(selStart, selEnd);
        final SelectedElementsIterator eli = new SelectedElementsIterator(doc.getDefaultRootElement(), Math.min(selStart, selEnd), max, SelectedElementsIterator.FORWARD_DIRECTION);
        int eStart, eEnd;
        Element elem;
        ImageIcon image;
        while ((elem = eli.next()) != null) {
            eStart = elem.getStartOffset();
            eEnd = elem.getEndOffset();
            if (elem.getName().equals(AbstractDocument.ContentElementName)) {
                if (eli.isFirstLeafElement()) {
                    clipContent.add(new StyledCopyContent(src.getText(selStart, (eli.isLastLeafElement()) ? selEnd - selStart : eEnd - selStart), elem.getAttributes()));
                } else {
                    clipContent.add(new StyledCopyContent(src.getText(eStart, (eli.isLastLeafElement()) ? selEnd - eStart : eEnd - eStart), elem.getAttributes()));
                }
            } else if (RiderStyles.isSupportedComponent(elem)) {
                clipContent.add(new StyledCopyContent(((SeparatorLine) (elem.getAttributes().getAttribute(StyleConstants.ComponentAttribute))).getLineType()));
            } else if ((image = RiderStyles.isImage(elem)) != null) {
                clipContent.add(new StyledCopyContent(image));
            }

        }
    }

    private static int findWordSeparator(final String text) {
        final char[] array = text.toCharArray();
        final int lenght = array.length;
        for (int i = 0; i < lenght; ++i)
            if (RiderEditorKit.WORD_SEPARATORS.get(array[i]))
                return i;
        return -1;
    }

    public static final int findNextWord(final RiderArea src) throws BadLocationException {
        final Document doc = src.getDocument();
        int index = src.getCaretPosition();
        final int selStart = index;
        final int selEnd = doc.getLength() - 1;
        if (index == selEnd)
            return index;
        int eStart, eEnd;
        final SelectedElementsIterator eli = new SelectedElementsIterator(doc.getDefaultRootElement(), index, doc.getLength() - 1, SelectedElementsIterator.FORWARD_DIRECTION);
        Element elem;
        String text;
        int result;
        while ((elem = eli.next()) != null) {
            eStart = elem.getStartOffset();
            eEnd = elem.getEndOffset();
            if (elem.getName().equals(AbstractDocument.ContentElementName)) {
                if (eli.isFirstLeafElement()) {
                    text = src.getText(selStart, (eli.isLastLeafElement()) ? selEnd - selStart : eEnd - selStart);
                } else {
                    text = src.getText(eStart, (eli.isLastLeafElement()) ? selEnd - eStart : eEnd - eStart);
                }
                if (!text.equals("")) {
                    if (eli.isFirstLeafElement() && text.length() > 2 && text.charAt(0) == ' ') {
                        text = text.substring(1);
                        ++index;
                    }
                    if ((result = findWordSeparator(text)) != -1)
                        return index += result;
                    else {
                        index += text.length();
                    }
                }
            } else if (RiderStyles.isSupportedComponent(elem)) {
                ++index;
                break;
            }

        }
        return index + 1;

    }


    /**
     * insert this <code>StyledContent</code> into a <code>Document</code>
     *
     * @param insertPos the position within the document to insert at
     */
    public final void insert(final JTextPane area, int insertPos) throws BadLocationException {
        String text;
        StyledCopyContent content;
        final RiderDocument doc = (RiderDocument) area.getDocument();

        for (final Iterator it = clipContent.iterator(); it.hasNext();) {
            content = (StyledCopyContent) it.next();
            switch (content.getContentType()) {
                case StyledCopyContent.CONTENT_TEXT:
                    text = (String) content.getContent();
                    if (!text.equals("")) {
                        if (RiderStyles.isReadonlySection(doc.getParagraphElement(insertPos))) {
                            doc.insertString(insertPos, "\n", null);
                            if (Utilities.getRowStart(area, insertPos) != insertPos) ++insertPos;
                        }
                        doc.insertString(insertPos, text, content.getAttr());
                        insertPos += text.length();
                    }
                    break;
                case StyledCopyContent.CONTENT_SEPARATOR:
                case StyledCopyContent.CONTENT_IMAGE:
                    if (RiderStyles.isReadonlySection(doc.getParagraphElement(insertPos))) {
                        doc.insertString(insertPos, "\n", null);
                        if (Utilities.getRowStart(area, insertPos) != insertPos) ++insertPos;
                    } else if (insertPos - 1 > 0 && !doc.getText(insertPos - 1, 1).equals("\n"))
                        doc.insertString(insertPos++, "\n", null);
                    if (content.getContentType() == StyledCopyContent.CONTENT_SEPARATOR)
                        doc.insertSeparateLine(area, insertPos++, ((Integer) content.getContent()).intValue());
                    else
                        doc.insertPicture(insertPos++, (Image) content.getContent());
                    break;
                default :
                    break;
            }
        }
    }

    public final String toString() {
        final StringBuffer buffer = new StringBuffer();
        for (final Iterator it = clipContent.iterator(); it.hasNext();)
            buffer.append(it.next().toString());
        return buffer.toString();
    }

}