/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.core;


import net.wordrider.utilities.Consts;
import net.wordrider.utilities.Utils;

import java.io.*;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLDecoder;
import java.text.MessageFormat;
import java.util.Properties;

/**
 * @author Vity
 */

public class AppPrefs {


    private static String DEFAULT_PROPERTIES = "wordrider.properties";
    private static final String CLASS_EXT = ".class";
    private static final String JAR_SEPARATOR = "!/";
    private static final String URL_SEPARATOR = "/";
    private static final String CLASS_SEPARATOR = ".";
    private static final String FILE_PREFIX = "file:";

    private static final Properties properties = loadDefaultProperties();
    private static String propertiesFile;
    private static final String DEFAULT_LANG_CODE = "EN";
    private static final String DEFAULT_NOT_FOUND = "Not found!";
    private static final String LANG_EXTENSION = ".properties";
    private static final String MNEMONIC_EXTENSION = ".mnemonic";
    private static final String HINT_EXTENSION = ".hint";


    private static Properties language = loadLangProperties();
    private static final String selectedLanguageKey = "selLanguage";
    private static String appPath = null;
    public static final String PROXY_USE = "settings.proxy.proxy";
    public static final String PROXY_URL = "settings.proxy.url";
    public static final String PROXY_SAVEPASSWORD = "settings.proxy.savepassword";
    public static final String PROXY_PORT = "settings.proxy.port";
    public static final String PROXY_LOGIN = "settings.proxy.login";
    public static final String PROXY_USERNAME = "settings.proxy.username";
    public static final String PROXY_PASSWORD = "settings.proxy.password";


    public static final String getSelectedLanguageCode() {
        return getProperty(selectedLanguageKey, DEFAULT_LANG_CODE);
    }

    public static final void setSelectedLanguageCode(final String code) {
        storeProperty(selectedLanguageKey, code);
    }

    private static Properties loadLangProperties() {
        return loadProperties(new StringBuffer(Consts.LANGUAGESDIR).append(getSelectedLanguageCode()).append(LANG_EXTENSION).toString(), true);
    }

    public static final void reloadLangProperties() {
        language = loadLangProperties();
    }

    private static Properties loadDefaultProperties() {
        return loadProperties(propertiesFile = new StringBuffer(getAppPath()).append(File.separator).append(DEFAULT_PROPERTIES).toString(), false);
    }

    public final static Properties loadProperties(final String propertiesFile, final boolean isResource) {
        Properties props = new Properties();
        InputStream inputStream = null;
        try {
            inputStream = (!isResource) ? new FileInputStream(propertiesFile) : ((URLClassLoader) AppPrefs.class.getClassLoader()).getResourceAsStream(propertiesFile);
            props.load(inputStream);
            inputStream.close();
            return props;
        } catch (IOException e) {
            try {
                if (inputStream != null)
                    inputStream.close();
            } catch (IOException ex) {
                Utils.processException(ex);
            }
            if (isWindows())
                Utils.log(Utils.ERROR_LOG, "Couldn't load properties:" + propertiesFile + " This is fatal error. Reinstal application may fix this problem.");
            Utils.processException(e);
            return props;
        }
    }

    public static final String getLabel(final String key) {
        return language.getProperty(key, DEFAULT_NOT_FOUND);
    }

    public static final String getHint(final String key) {
        return language.getProperty(key + HINT_EXTENSION, key);
    }

    public static final String getLabel(final String key, Object parameter) {
        return getLabel(key, new Object[]{(parameter == null ? "" : parameter)});
    }

    public static final String getLabel(final String key, final Object[] parameters) {
        return new MessageFormat(language.getProperty(key, DEFAULT_NOT_FOUND)).format(parameters);
    }

    public static final char getMnemonic(final String key) {
        final String mnemonic = language.getProperty(key + MNEMONIC_EXTENSION, "");
        return (mnemonic.length() > 0) ? mnemonic.charAt(0) : '\0';
    }


    public static final String getProperty(final String key, final String defaultValue) {
        return properties.getProperty(key, defaultValue);
    }

    public static final int getProperty(final String key, final int defaultValue) {
        try {
            return Integer.parseInt(properties.getProperty(key, String.valueOf(defaultValue)));
        } catch (NumberFormatException e) {
            return defaultValue;
        }
    }

    public static final boolean getProperty(final String key, final boolean defaultValue) {
        String property = properties.getProperty(key, String.valueOf(defaultValue));
        return Boolean.valueOf(property).booleanValue();
    }

    public static final Object removeProperty(final String key) {
        return properties.remove(key);
    }

    public static final void store() {
        if (!getProperty(PROXY_SAVEPASSWORD, false)) {
            properties.remove(PROXY_PASSWORD);
        }
        OutputStream outputStream = null;
        try {
            outputStream = new FileOutputStream(propertiesFile);
            properties.store(outputStream, Consts.APPVERSION + System.getProperty("line.separator", "\n") + "#WordRider properties. Only for experienced users.");
            outputStream.close();
        } catch (IOException e) {
            try {
                if (outputStream != null)
                    outputStream.close();
            } catch (IOException ex) {
                Utils.processException(ex);
            }
            Utils.log(Utils.ERROR_LOG, "Couldn't save app properties. This is a fatal error. Please reinstall the application.");
            Utils.processException(e);
        }
        Utils.log(Utils.INFO_LOG, "Properties were saved successfuly");
    }

    public static final void storeProperty(final String key, final boolean value) {
        properties.setProperty(key, String.valueOf(value));
    }

    public static final void storeProperty(final String key, final String value, final boolean store) {
        properties.setProperty(key, value);
        if (store)
            store();
    }

    public static final void storeProperty(final String key, final String value) {
        properties.setProperty(key, value);
    }

    public static boolean isWindows() {
        final String osName = System.getProperty("os.name");
        return (osName == null || osName.startsWith("Windows")) ? true : false;
    }

    public static String getAppPath() {
        if (appPath != null)
            return appPath;
        int end = 0;
        String urlStr = null;
        String clsName = AppPrefs.class.getName();
        int clsNameLen = clsName.length() + CLASS_EXT.length();
        int pos = clsName.lastIndexOf(CLASS_SEPARATOR);
        final boolean debug = Utils.isDefinedForLog(Utils.DEBUG_LOG);
        if (pos > -1) {
            clsName = clsName.substring(pos + 1);
        }
        if (debug)
            Utils.log(Utils.DEBUG_LOG, "ClassName " + clsName + CLASS_EXT);
        URL url = AppPrefs.class.getResource(clsName + CLASS_EXT);
        if (debug)
            Utils.log(Utils.DEBUG_LOG, "URL " + url);
        if (url != null) {
            urlStr = url.toString();
            if (debug)
                Utils.log(Utils.DEBUG_LOG, "Url string1 " + urlStr);
            pos = urlStr.indexOf(JAR_SEPARATOR);
            if (pos > -1) {
                urlStr = urlStr.substring(0, pos);
                if (debug)
                    Utils.log(Utils.DEBUG_LOG, "URL String2 " + urlStr);
                end = urlStr.lastIndexOf(URL_SEPARATOR) + 1;
            } else {
                end = urlStr.length() - clsNameLen;
            }
            pos = urlStr.lastIndexOf(FILE_PREFIX);
            if (pos > -1) {
                pos += FILE_PREFIX.length() + (isWindows() ? 1 : 0);
            } else {
                pos = 0;
            }
            urlStr = urlStr.substring(pos, end);
            if (Utils.isDefinedForLog(Utils.DEBUG_LOG))
                Utils.log(Utils.DEBUG_LOG, "App Path is " + urlStr);
            String decoded = "";

            try {
                decoded = URLDecoder.decode(urlStr, "ISO-8859-1");
            } catch (UnsupportedEncodingException e) {
                Utils.processException(e);
            }
            return appPath = decoded;
        }
        return appPath = "";

    }
}