/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.dialogs;

import net.wordrider.area.RiderEditorKit;
import net.wordrider.area.RiderStyles;
import net.wordrider.core.AppPrefs;
import net.wordrider.utilities.Swinger;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public final class FindReplaceDialog extends JDialog {

    private int mode;

    private JEditorPane editor;

    private Document doc;

    private String searchText, phrase, newPhrase;


    private int lastPos, offset, replaceDiff;

    private boolean findInProgress = false;

    private int operation;

    private int replaceChoice;

    private final Vector listeners = new Vector(0);

    private static final Object[] replaceOptions = {AppPrefs.getLabel("dialog.findr.btnYes"), AppPrefs.getLabel("dialog.findr.btnNo"), AppPrefs.getLabel("dialog.findr.btnAll"), AppPrefs.getLabel("dialog.findr.btnDone")};

    /* Constants for method toggleState */
    private static final boolean STATE_LOCKED = false;
    private static final boolean STATE_UNLOCKED = true;

    /* Constants for replaceOptions */
    public static final int RO_YES = 0;
    public static final int RO_NO = 1;
    private static final int RO_ALL = 2;
    private static final int RO_DONE = 3;

    /* Constants for dialog mode */
    private static final int MODE_DOCUMENT = 1;
    private static final int MODE_PROJECT = 2;

    /* Constants for operation */
    private static final int OP_NONE = 0;
    private static final int OP_FIND = 1;
    private static final int OP_REPLACE = 2;

    /* ---- GUI elements start ---------*/

    private TitledBorder titledBorder1;
    private final JButton btnFindNext = Swinger.getRiderButton("dialog.findr.findNext");
    private final JCheckBox jcbStartOnTop = Swinger.getRiderCheckBox("dialog.findr.fromStart");
    private final JRadioButton radioButtonDown = new JRadioButton();
    private final JCheckBox jcbWholeWords = Swinger.getRiderCheckBox("dialog.findr.whole");
    private final JPanel panelBtn = new JPanel();
    private final JPanel panelOptions = new JPanel();
    private final JPanel panelFind = new JPanel();
    private JComboBox textFieldReplace;
    private final JPanel panelMain = new JPanel();
    private final JRadioButton radioButtonUp = new JRadioButton();
    private JComboBox textFieldPhrase;
    private final JCheckBox jcbMatchCase = Swinger.getRiderCheckBox("dialog.findr.case");
    private final JLabel jLabel3 = new JLabel();
    private final JLabel jLabel4 = new JLabel();
    private final GridBagLayout gridBagLayout4 = new GridBagLayout();
    private final GridBagLayout gridBagLayout5 = new GridBagLayout();
    private final JButton btnClose = Swinger.getRiderButton("dialog.findr.closeBtn");
    private final GridBagLayout gridBagLayout6 = new GridBagLayout();
    private final JButton btnReplace = Swinger.getRiderButton("dialog.findr.replaceBtn");
    private final JButton btnCancel = Swinger.getRiderButton("dialog.findr.cancelBtn");
//    private final JCheckBox jcbUnused = Swinger.getRiderCheckBox();
    private final JCheckBox jcbProject = new JCheckBox();

    private int replaceCount;

    private final static String PHRASE_PREFERENCES = "findPhrase", REPLACE_PREFERENCES = "replacePhrase";
    private final static int MAX_RECENT_PHRASES_COUNT = 5;


    public FindReplaceDialog(final Frame owner, final JEditorPane editor) {
        super(owner, true);
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        setEditor(editor);
        setMode(MODE_DOCUMENT);
        //owner.setIconImage(Swinger.getCachedIcon("find.gif").getImage());
        try {
            init();
            initDialogContents();
            centerDialog(owner);
            pack();
            setVisible(true);
        } catch (Exception e) {
            Utils.processException(e);
        }
    }

    protected void processWindowEvent(WindowEvent e) {
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            Utils.logDebug("Saving find/replace strings to preferences");
            saveUsedPhrases();
        }
        super.processWindowEvent(e);    //call to super
    }

    public final void addFindReplaceListener(final FindReplaceListener listener) {
        listeners.addElement(listener);
    }

    public final void removeFindReplaceListener(final FindReplaceListener listener) {
        listeners.removeElement(listener);
    }

    protected final JRootPane createRootPane() {
        final ActionListener actionListener = new ActionListener() {
            public void actionPerformed(final ActionEvent actionEvent) {
                if (btnCancel.isEnabled())
                    btnCancel_actionPerformed();
                else
                    btnClose_actionPerformed();
            }
        };
        final JRootPane rootPane = new JRootPane();
        final KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
        rootPane.registerKeyboardAction(actionListener, stroke, JComponent.WHEN_IN_FOCUSED_WINDOW);
        return rootPane;
    }


    private void fireGetNextDocument() {
        final Enumeration listenerList = listeners.elements();
        while (listenerList.hasMoreElements()) {
            ((FindReplaceListener) listenerList.nextElement()).getNextDocument(new FindReplaceEvent(this));
        }
    }

    private void fireGetFirstDocument() {
        final Enumeration listenerList = listeners.elements();
        while (listenerList.hasMoreElements()) {
            ((FindReplaceListener) listenerList.nextElement()).getFirstDocument(new FindReplaceEvent(this));
        }
    }

    public final void resumeOperation() {
        this.doc = editor.getDocument();
        findInProgress = false;
        initFind();
        switch (operation) {
            case OP_FIND:
                find();
                break;
            case OP_REPLACE:
                replace();
                break;
        }
    }

    private void terminateOperation() {
        switch (operation) {
            case OP_FIND:
                message(AppPrefs.getLabel("dialog.findr.notfound"));
                toggleState(STATE_UNLOCKED);
                btnReplace.setEnabled(true);
                break;
            case OP_REPLACE:
                switch (replaceChoice) {
                    case RO_YES:
                    case RO_NO:
                        message(AppPrefs.getLabel("dialog.findr.notfound"));
                        break;
                    case RO_ALL:
                        message(AppPrefs.getLabel("dialog.findr.replaceCount", new Integer(replaceCount)));
                        break;
                }
                toggleState(STATE_UNLOCKED);
                editor.firePropertyChange("undoredo", true, false);
                setVisible(true);
                break;
        }
        operation = OP_NONE;
    }

    private void btnFindNext_actionPerformed() {
        operation = OP_FIND;
        btnReplace.setEnabled(false);
        if (mode == MODE_PROJECT &&
                jcbProject.isSelected() &&
                listeners.size() > 0 &&
                !findInProgress) {
            fireGetFirstDocument();
        } else {
            initFind();
            find();
        }
    }

    private void btnReplace_actionPerformed() {
        operation = OP_REPLACE;
        replaceChoice = RO_YES;
        setVisible(false);
        if (mode == MODE_PROJECT &&
                jcbProject.isSelected() &&
                listeners.size() > 0) {
            fireGetFirstDocument();
        } else {
            initFind();
            replace();
        }
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                textFieldReplace.requestFocus();
            }
        });

    }

    private void btnCancel_actionPerformed() {
        toggleState(STATE_UNLOCKED);
        btnReplace.setEnabled(true);
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                textFieldPhrase.requestFocus();
            }
        });
    }

    private void storeProperties(final String keyProperties, final JComboBox combo) {
        final Collection collection = ((MyComboDefaultModel) combo.getModel()).getList();
        int counter = collection.size() - 1;
        for (final Iterator it = collection.iterator(); it.hasNext(); --counter) {
            AppPrefs.storeProperty(keyProperties + counter, it.next().toString());
        }
    }

    private void saveUsedPhrases() {
        storeProperties(PHRASE_PREFERENCES, textFieldPhrase);
        storeProperties(REPLACE_PREFERENCES, textFieldReplace);
    }

    private void btnClose_actionPerformed() {
        saveUsedPhrases();
        //result = JOptionPane.OK_OPTION;
        dispose();
    }

    private void setEditor(final JEditorPane editor) {
        this.editor = editor;
        this.doc = editor.getDocument();
    }

    private void setMode(final int mode) {
        this.mode = mode;
        if (mode == MODE_PROJECT) {
            jcbProject.setVisible(true);
        } else {
            jcbProject.setVisible(false);
        }
    }

    private void initFind() {
        if (!findInProgress) {
            try {
                searchText = doc.getText(0, doc.getLength());
            } catch (Exception e) {
                Utils.processException(e);
            }
            phrase = textFieldPhrase.getEditor().getItem().toString();
            ((MyComboDefaultModel) textFieldPhrase.getModel()).addElement(phrase);
            newPhrase = textFieldReplace.getEditor().getItem().toString();
            ((MyComboDefaultModel) textFieldReplace.getModel()).addElement(newPhrase);
            replaceDiff = newPhrase.length() - phrase.length();
            offset = 0;
            if (!jcbMatchCase.isSelected()) {
                phrase = phrase.toLowerCase();
                searchText = searchText.toLowerCase();
            }
            if (jcbStartOnTop.isSelected()) {
                if (radioButtonUp.isSelected()) {
                    lastPos = doc.getLength();
                } else {
                    lastPos = 0;
                }
            } else {
                lastPos = editor.getSelectionStart();
            }
            toggleState(STATE_LOCKED);
        }
    }

    private void find() {
        if (!doFind()) {
            if (mode == MODE_PROJECT &&
                    jcbProject.isSelected() &&
                    listeners.size() > 0) {
                fireGetNextDocument();
            } else {
                terminateOperation();
            }
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    textFieldPhrase.requestFocus();
                }
            });
        } else {
            btnFindNext.setEnabled(true);
            btnFindNext.requestFocus();
        }
    }

    private boolean doFind() {
        boolean found = false;
        int start = findNext();
        if (jcbWholeWords.isSelected()) {
            start = findWholeWords(start);
        }
        if (start >= 0) {
            lastPos = start;
            if (radioButtonDown.isSelected()) {
                start += offset;
            }
            editor.select(start, start + phrase.length());
            found = true;
        }
        return found;
    }

    private int findNext() {
        final int start;
        if (radioButtonUp.isSelected()) {
            if (lastPos < doc.getLength()) {
                start = searchText.lastIndexOf(phrase, lastPos - 1);
            } else {
                start = searchText.lastIndexOf(phrase, lastPos);
            }
        } else {
            if (lastPos > 0) {
                start = searchText.indexOf(phrase, lastPos + phrase.length());
            } else {
                start = searchText.indexOf(phrase, lastPos);
            }
        }
        return start;
    }

    private int findWholeWords(int start) {
        while ((start > 0) &&
                ((!RiderEditorKit.WORD_SEPARATORS.get(searchText.charAt(start - 1))) ||
                (!RiderEditorKit.WORD_SEPARATORS.get(searchText.charAt(start + phrase.length()))))) {
            lastPos = start;
            start = findNext();
        }
        return start;
    }


    private void replace() {
        editor.firePropertyChange("undoredo", false, true);
        while (doFind() && replaceChoice != RO_DONE) {
            if (replaceChoice != RO_ALL)
                replaceChoice = getReplaceChoice();
            switch (replaceChoice) {
                case RO_YES:
                    replaceOne();
                    break;
                case RO_ALL:
                    replaceOne();
                    replaceCount = 1;
                    while (doFind()) {
                        replaceOne();
                        ++replaceCount;
                    }
                    break;
            }
        }
        if (mode == MODE_PROJECT &&
                jcbProject.isSelected() &&
                listeners.size() > 0) {
            switch (replaceChoice) {
                case RO_YES:
                case RO_NO:
                case RO_ALL:
                    fireGetNextDocument();
                    break;
                case RO_DONE:
                    terminateOperation();
                    break;
            }
        } else
            terminateOperation();
    }

    private int getReplaceChoice() {
        return JOptionPane.showOptionDialog(this,
                AppPrefs.getLabel("dialog.findr.confirmR", phrase),
                AppPrefs.getLabel("dialog.findr.title"),
                JOptionPane.YES_NO_CANCEL_OPTION,
                JOptionPane.QUESTION_MESSAGE,
                null,
                replaceOptions,
                null);
    }

    private void replaceOne() {
        editor.replaceSelection(newPhrase);
        offset += replaceDiff;
    }

    private void initDialogContents() {
        btnCancel.setEnabled(false);
        radioButtonUp.setSelected(false);
        radioButtonDown.setSelected(true);
        jcbWholeWords.setSelected(false);
        jcbMatchCase.setSelected(false);
        jcbStartOnTop.setSelected(true);
        jcbProject.setSelected(false);
        textFieldPhrase.getEditor().setItem(editor.getSelectedText());
        btnFindNext.setEnabled(!isEmpty(textFieldPhrase));
        btnReplace.setEnabled(btnFindNext.isEnabled());
        textFieldReplace.getEditor().setItem("");
    }

    private void centerDialog(final Frame owner) {
        final Dimension dlgSize = getPreferredSize();
        final Dimension frmSize = owner.getSize();
        final Point loc = owner.getLocation();
        setLocation((frmSize.width - dlgSize.width) / 2 + loc.x + 150,
                (frmSize.height - dlgSize.height) / 2 + loc.y - 50);
    }

    private void toggleState(final boolean unlocked) {
        btnCancel.setEnabled(!unlocked);
        btnFindNext.setEnabled(unlocked && !isEmpty(textFieldPhrase));
        textFieldPhrase.setEnabled(unlocked);
        textFieldReplace.setEnabled(unlocked);
        jLabel3.setEnabled(unlocked);
        jLabel4.setEnabled(unlocked);
        jcbWholeWords.setEnabled(unlocked);
        jcbMatchCase.setEnabled(unlocked);
        jcbStartOnTop.setEnabled(unlocked);
        radioButtonUp.setEnabled(unlocked);
        radioButtonDown.setEnabled(unlocked);
        jcbProject.setEnabled(unlocked);
        findInProgress = !unlocked;
    }

    private void message(final String msgText) {
        JOptionPane.showMessageDialog(this,
                msgText,
                AppPrefs.getLabel("dialog.findr.title"),
                JOptionPane.INFORMATION_MESSAGE);
    }

    /* ----------- Helper methods end ------- */

    private static boolean isEmpty(final JComboBox field) {
        Object item = field.getEditor().getItem();
        return item == null || item.equals("");
    }

    private final class ActionKeyListener implements KeyListener {
        public void keyPressed(final KeyEvent e) {
            final JTextField field = ((JTextField) e.getSource());
            if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                if (field.equals(textFieldPhrase)) {
                    btnFindNext_actionPerformed();
                } else
                    btnReplace_actionPerformed();
                e.consume();
            }
        }

        public void keyReleased(final KeyEvent e) {
        }

        public void keyTyped(final KeyEvent e) {
        }
    }

       private final class InputFieldsMethodListener implements DocumentListener {
        public void inputMethodTextChanged() {
            final boolean isEmpty = isEmpty(textFieldPhrase);
            if (btnFindNext.isEnabled() != !isEmpty)
                btnFindNext.setEnabled(!isEmpty);
            if (btnReplace.isEnabled() != !isEmpty)
                btnReplace.setEnabled(!isEmpty);
        }

        public void changedUpdate(final DocumentEvent e) {
            inputMethodTextChanged();
        }

        public void insertUpdate(final DocumentEvent e) {
            inputMethodTextChanged();
        }

        public void removeUpdate(final DocumentEvent e) {
            inputMethodTextChanged();
        }
    }

    /**
     * GUI builder init
     */
    private void init() {
        textFieldPhrase = new JComboBox(loadSearchUsedList(PHRASE_PREFERENCES));
        textFieldReplace = new JComboBox(loadSearchUsedList(REPLACE_PREFERENCES));
        textFieldPhrase.setEditable(true);
        textFieldReplace.setEditable(true);

        // textFieldReplace.getEditor().addActionListener();
        textFieldPhrase.setMaximumRowCount(MAX_RECENT_PHRASES_COUNT);
        textFieldReplace.setMaximumRowCount(MAX_RECENT_PHRASES_COUNT);
        titledBorder1 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white, new Color(142, 142, 142)), AppPrefs.getLabel("dialog.findr.options"));
        final ButtonGroup bgSearchDirection = new ButtonGroup();
        btnFindNext.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(final ActionEvent e) {
                btnFindNext_actionPerformed();
            }
        });
        final Dimension buttonSize = new Dimension(90, 25);
        btnFindNext.setPreferredSize(buttonSize);
        btnFindNext.setMinimumSize(buttonSize);
        btnFindNext.setMaximumSize(buttonSize);

        radioButtonDown.setText(AppPrefs.getLabel("dialog.findr.searchDown"));
        radioButtonDown.setMnemonic(AppPrefs.getMnemonic("dialog.findr.searchDown"));
        panelBtn.setLayout(gridBagLayout4);
        panelOptions.setBorder(titledBorder1);
        panelOptions.setLayout(new GridBagLayout());
        panelOptions.setPreferredSize(new Dimension(305, 88));
        panelFind.setLayout(gridBagLayout5);

        textFieldReplace.setMinimumSize(new Dimension(4, 12));
        textFieldReplace.setPreferredSize(new Dimension(30, 12));
        //textFieldReplace.setText("textFieldReplace");
        panelMain.setLayout(gridBagLayout6);
        radioButtonUp.setText(AppPrefs.getLabel("dialog.findr.searchup"));
        radioButtonUp.setMnemonic(AppPrefs.getMnemonic("dialog.findr.searchup"));
        //btnClose.setRolloverEnabled(true);

        textFieldPhrase.setMinimumSize(new Dimension(4, 12));
        textFieldPhrase.setPreferredSize(new Dimension(44, 12));
        final FocusListener focusListener = new Swinger.SelectAllOnFocusListener();
        final KeyListener keyListener = new ActionKeyListener();
        textFieldPhrase.addKeyListener(keyListener);
        Component editorComponent = textFieldPhrase.getEditor().getEditorComponent();
        if (editorComponent instanceof JTextComponent) {
            final JTextComponent component = (JTextComponent)editorComponent;
            component.addKeyListener(keyListener);
            component.getDocument().addDocumentListener(new InputFieldsMethodListener());
            component.addFocusListener(focusListener);
            Utils.addKeyActions(component);
        }
        editorComponent = textFieldReplace.getEditor().getEditorComponent();
        if (editorComponent instanceof JTextComponent) {
            final JTextComponent component = (JTextComponent)editorComponent;
            component.addKeyListener(keyListener);
            component.addFocusListener(focusListener);
            Utils.addKeyActions(component);
        }
        final Font font = RiderStyles.getAreaFont().deriveFont(Font.PLAIN, 12);
        textFieldPhrase.setFont(font);
        textFieldReplace.setFont(font);
        jLabel3.setText(AppPrefs.getLabel("dialog.findr.replacewith"));
        jLabel4.setText(AppPrefs.getLabel("dialog.findr.textfind"));
        jLabel3.setLabelFor(textFieldPhrase);
        jLabel3.setDisplayedMnemonic(AppPrefs.getMnemonic("dialog.findr.replacewith"));
        jLabel4.setDisplayedMnemonic(AppPrefs.getMnemonic("dialog.findr.textfind"));
        jLabel4.setLabelFor(textFieldReplace);
        btnClose.setPreferredSize(buttonSize);
        btnClose.setMaximumSize(buttonSize);
        btnClose.setMinimumSize(buttonSize);

        btnClose.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(final ActionEvent e) {
                btnClose_actionPerformed();
            }
        });
        this.setModal(true);
        this.setTitle(AppPrefs.getLabel("dialog.findr.title"));
        btnReplace.setPreferredSize(buttonSize);
        btnReplace.setMaximumSize(buttonSize);
        btnReplace.setMinimumSize(buttonSize);

        btnReplace.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(final ActionEvent e) {
                btnReplace_actionPerformed();
            }
        });
        btnCancel.setPreferredSize(buttonSize);
        btnCancel.setMaximumSize(buttonSize);
        btnCancel.setMinimumSize(buttonSize);

        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(final ActionEvent e) {
                btnCancel_actionPerformed();
            }
        });
        //  jcbUnused.setText("jcbUnused");
        //jcbUnused.setVisible(false);
        this.getContentPane().add(panelMain, BorderLayout.NORTH);
        panelBtn.add(btnFindNext, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(4, 4, 0, 4), 0, 0));
        panelBtn.add(btnClose, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
                , GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(0, 4, 4, 4), 0, 0));
        panelBtn.add(btnReplace, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
                , GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
        panelBtn.add(btnCancel, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
                , GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
        panelMain.add(panelFind, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
                , GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(4, 4, 4, 4), 0, 0));
        panelMain.add(panelBtn, new GridBagConstraints(1, 0, 1, 2, 1.0, 1.0
                , GridBagConstraints.NORTHEAST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
        panelFind.add(textFieldPhrase, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(4, 4, 0, 0), 0, 12));
        panelFind.add(jLabel4, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
        panelFind.add(textFieldReplace, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(4, 4, 0, 0), 0, 12));
        panelFind.add(jLabel3, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
                , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
        panelMain.add(panelOptions, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 4, 0, 0), 0, 0));

        panelOptions.add(jcbWholeWords, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(-10, 0, 0, 0), 10, 0));
        panelOptions.add(jcbMatchCase, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(-6, 0, -5, 0), 10, 0));
        panelOptions.add(jcbStartOnTop, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(-3, 0, -5, 0), 10, 0));
        panelOptions.add(radioButtonUp, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(-10, 0, 0, 0), 0, 0));
        panelOptions.add(radioButtonDown, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(-3, 0, 0, 0), 0, 0));
        //  panelOptions.add(jcbUnused, null);
        //   panelOptions.add(jcbProject, null);
        bgSearchDirection.add(radioButtonUp);
        bgSearchDirection.add(radioButtonDown);
    }


    private class MyComboDefaultModel extends DefaultComboBoxModel {
        private Stack stack;

        public MyComboDefaultModel(Stack v) {
            super(v);    //call to super
            this.stack = v;
        }

        public final void addElement(Object anObject) {
            if (!anObject.equals("") && getIndexOf(anObject) < 0) {
                super.insertElementAt(anObject, 0);
                if (stack.size() > MAX_RECENT_PHRASES_COUNT)
                    super.removeElementAt(MAX_RECENT_PHRASES_COUNT - 1);
            }
        }

        public final Collection getList() {
            return stack;
        }
    }

    private ComboBoxModel loadSearchUsedList(final String keyProperties) {
        String searched, key;
        final Stack collection = new Stack();
        for (int counter = 0; (searched = AppPrefs.getProperty(key = keyProperties + counter, null)) != null; ++counter) {
            if (searched.length() > 0) {
                collection.add(0, searched);
                AppPrefs.removeProperty(key);
            }
        }
        return new MyComboDefaultModel(collection);
    }
}