/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.dialogs;

import net.wordrider.core.AppPrefs;
import net.wordrider.ti89.TITextFileInfo;
import net.wordrider.utilities.Consts;
import net.wordrider.utilities.Swinger;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusListener;

/**
 * @author Vity
 */
public final class SaveSettingsDialog extends AppDialog {
    private final JTextField inputFolder = new JTextField();
    private final JTextField inputVariable = new JTextField();
    private final JTextField inputComment = new JTextField();
    private final JPanel picturePanel = new JPanel();
    private final JButtonGroup buttonGroup = new JButtonGroup();
    private JRadioButton radioDontChange;
    private JRadioButton radioUseLast;
    private JRadioButton radioSameAsDocument;
    private JRadioButton radioSetNew;
    private final JTextField inputPictureFolder = new JTextField();
    private final JPanel btnPanel = new JPanel();
    private JButton btnCancel;
    private JButton btnSave;
    private final JCheckBox checkSave = Swinger.getRiderCheckBox("savesettings.pictures");

    private final static int RESULT_SAVE = 0;
    private final static int RESULT_CANCEL = 1;
    private int result = RESULT_SAVE;
    private TITextFileInfo fileInfo;
    private Frame frame;
    private static final String DEFAULT_FOLDER_NAME = "main";

    public SaveSettingsDialog(final Frame owner, final TITextFileInfo fileInfo) {
        super(owner, true);
        this.frame = owner;
        try {
            init();
        } catch (Exception e) {
            Utils.processException(e);
        }
        initDialogContents(fileInfo);
        Swinger.centerDialog(owner, this);
        this.setModal(true);
        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.setTitle(AppPrefs.getLabel("savesettings.title"));
        this.setVisible(true);
    }


    private void initDialogContents(final TITextFileInfo fileInfo) {
        Utils.addKeyActions(inputFolder);
        Utils.addKeyActions(inputVariable);
        Utils.addKeyActions(inputComment);
        Utils.addKeyActions(inputPictureFolder);
        if (fileInfo.getFolderName().equals("")) {
            inputFolder.setText(DEFAULT_FOLDER_NAME);
            inputComment.setText("created by (c)" + Consts.APPVERSION);
        } else {
            inputFolder.setText(fileInfo.getFolderName());
            inputComment.setText(fileInfo.getComment());
        }
        inputVariable.setText(fileInfo.getVarName());
        inputPictureFolder.setText(fileInfo.getPictureFolder());

        switch (fileInfo.getPictureProcessingType()) {
            case TITextFileInfo.PICTURE_DONTSAVE:
                checkSave.setSelected(false);
                break;
            case TITextFileInfo.PICTURE_USESAMEASFORDOCUMENT:
                radioSameAsDocument.setSelected(true);
                break;
            case TITextFileInfo.PICTURE_FOLDER_DONTCHANGE:
                radioDontChange.setSelected(true);
                break;
            case TITextFileInfo.PICTURE_FOLDER_USELAST:
                radioUseLast.setSelected(true);
                break;
            case TITextFileInfo.PICTURE_FOLDER_USEOWN:
                radioSetNew.setSelected(true);
                break;
        }
        checkSave.setSelected(fileInfo.getPictureProcessingType() != TITextFileInfo.PICTURE_DONTSAVE);
        checkSave_actionPerformed();
        inputPictureFolder.setEnabled(radioSetNew.isSelected());
        Swinger.inputFocus(inputFolder);
    }

    public TITextFileInfo getResult() {
        if (result == RESULT_CANCEL)
            return null;
        else
            return fileInfo;
    }

    void btnCancel_actionPerformed() {
        dispose();
    }

    private boolean checkNameValidity(final JTextField field, final boolean digitStart, final int maxLenght) {
        boolean result = true;
        final String text = field.getText();
        if ((text == null || text.equals("") && !field.equals(inputComment))) {
            Swinger.showErrorDialog(frame, AppPrefs.getLabel("savesettings.mustbeset"));
            result = false;
        } else if (digitStart && Character.isDigit(text.charAt(0))) {
            Swinger.showErrorDialog(frame, AppPrefs.getLabel("savesettings.startsdigit"));
            result = false;
        } else if (text.length() > maxLenght) {
            Swinger.showErrorDialog(frame, AppPrefs.getLabel("savesettings.toolong", String.valueOf(maxLenght)));
            result = false;
        }

        if (!result)
            Swinger.inputFocus(field);
        return result;
    }

    void btnSave_actionPerformed() {
        if (!checkNameValidity(inputFolder, true, 8) || !checkNameValidity(inputVariable, true, 8) || !checkNameValidity(inputComment, false, 40))
            return;
        fileInfo = new TITextFileInfo();
        fileInfo.setComment(inputComment.getText());
        fileInfo.setFolderName(inputFolder.getText());
        fileInfo.setVarName(inputVariable.getText());
        final int processingType;
        if (!checkSave.isSelected())
            processingType = TITextFileInfo.PICTURE_DONTSAVE;
        else {
            final AbstractButton button = buttonGroup.getSelected();
            if (button.equals(radioDontChange))
                processingType = TITextFileInfo.PICTURE_FOLDER_DONTCHANGE;
            else if (button.equals(radioUseLast))
                processingType = TITextFileInfo.PICTURE_FOLDER_USELAST;
            else if (button.equals(radioSameAsDocument))
                processingType = TITextFileInfo.PICTURE_USESAMEASFORDOCUMENT;
            else if (button.equals(radioSetNew)) {
                if (!checkNameValidity(inputPictureFolder, true, 8))
                    return;
                processingType = TITextFileInfo.PICTURE_FOLDER_USELAST;
                fileInfo.setPictureFolder(inputPictureFolder.getText());
            } else
                processingType = TITextFileInfo.PICTURE_FOLDER_DONTCHANGE;
        }

        fileInfo.setPictureProcessingType(processingType);
        dispose();
    }

    void checkSave_actionPerformed() {
        final boolean isEnabled = checkSave.isSelected();
        radioDontChange.setEnabled(isEnabled);
        radioSetNew.setEnabled(isEnabled);
        radioUseLast.setEnabled(isEnabled);
        radioSameAsDocument.setEnabled(isEnabled);
    }


    private static class LimitedPlainDocument extends javax.swing.text.PlainDocument {

        private int maxLen = -1;
        private boolean canStartWithDigit;

        private boolean isAcceptable(final char[] text) {
            final int length = text.length;
            for (int i = 0; i < length; ++i)
                if (!(Character.isLetter(text[i]) || Character.isDigit(text[i])))
                    return false;
            return true;
        }

        public LimitedPlainDocument(final int maxLen, final boolean canStartWithDigit) {
            this.maxLen = maxLen;
            this.canStartWithDigit = canStartWithDigit;
        }

        public final void insertString(final int param, final String str, final javax.swing.text.AttributeSet attributeSet) throws javax.swing.text.BadLocationException {
            if (str != null && ((maxLen > 0 && this.getLength() + str.length() > maxLen) || (!canStartWithDigit && getLength() == 0 && str.length() > 0 && Character.isDigit(str.charAt(0))) || !isAcceptable(str.toCharArray()))) {
                java.awt.Toolkit.getDefaultToolkit().beep();
                return;
            }
            super.insertString(param, str, attributeSet);
        }

    }

    private class ActionButtonsAdapter implements java.awt.event.ActionListener {

        public void actionPerformed(final ActionEvent e) {
            if (e.getSource().equals(btnSave))
                btnSave_actionPerformed();
            else
                btnCancel_actionPerformed();
        }
    }

    private class CheckSaveAdapter implements java.awt.event.ActionListener {

        public void actionPerformed(final ActionEvent e) {
            checkSave_actionPerformed();
        }
    }

    private class RadiosAdapter implements java.awt.event.ActionListener {

        public void actionPerformed(final ActionEvent e) {
            boolean setNew = (e.getSource().equals(radioSetNew));
            inputPictureFolder.setEnabled(setNew);
            if (setNew)
                Swinger.inputFocus(inputPictureFolder);
        }
    }

    private void init() throws Exception {
        inputFolder.setDocument(new LimitedPlainDocument(8, false));
        inputVariable.setDocument(new LimitedPlainDocument(8, false));
        inputComment.setDocument(new LimitedPlainDocument(40, true));
        inputPictureFolder.setDocument(new LimitedPlainDocument(8, false));
        final FocusListener focusListener = new Swinger.SelectAllOnFocusListener();
        inputFolder.addFocusListener(focusListener);
        inputVariable.addFocusListener(focusListener);
        inputComment.addFocusListener(focusListener);
        inputPictureFolder.addFocusListener(focusListener);

        //inputVariable.setPreferredSize(new Dimension(70,-1));
        final Container mainPanel = this.getContentPane();
        mainPanel.setLayout(new GridBagLayout());
        final JLabel jLabel1 = new JLabel(AppPrefs.getLabel("savesettings.input.folder"));
        jLabel1.setDisplayedMnemonic(AppPrefs.getMnemonic("savesettings.input.folder"));
        jLabel1.setLabelFor(inputFolder);
        final JLabel jLabel2 = new JLabel(AppPrefs.getLabel("savesettings.input.variable"));
        jLabel2.setDisplayedMnemonic(AppPrefs.getMnemonic("savesettings.input.variable"));
        jLabel2.setLabelFor(inputVariable);
        final JLabel jLabel3 = new JLabel(AppPrefs.getLabel("savesettings.input.comment"));
        jLabel3.setDisplayedMnemonic(AppPrefs.getMnemonic("savesettings.input.comment"));
        jLabel3.setLabelFor(inputComment);
        picturePanel.setEnabled(true);
        picturePanel.setBorder(new TitledBorder(BorderFactory.createLineBorder(Color.gray, 1), AppPrefs.getLabel("savesettings.pictures.folder")));
        picturePanel.setLayout(new GridBagLayout());
        radioDontChange = Swinger.getRiderRadio("savesettings.pictures.samesource");
        radioSameAsDocument = Swinger.getRiderRadio("savesettings.pictures.samefolder");
        radioUseLast = Swinger.getRiderRadio("savesettings.pictures.lastfolder");
        radioSetNew = Swinger.getRiderRadio("savesettings.pictures.newfolder");

        radioSameAsDocument.addActionListener(new RadiosAdapter());
        radioDontChange.addActionListener(new RadiosAdapter());
        radioUseLast.addActionListener(new RadiosAdapter());
        radioSetNew.addActionListener(new RadiosAdapter());
        btnPanel.setLayout(new GridBagLayout());
        final Dimension buttonSize = new Dimension(90, 25);
        btnCancel = Swinger.getRiderButton("savesettings.cancelBtn");
        btnCancel.setMinimumSize(buttonSize);
        btnCancel.setPreferredSize(buttonSize);
        btnCancel.setMaximumSize(buttonSize);
        btnSave = Swinger.getRiderButton("savesettings.saveBtn");
        btnSave.setMinimumSize(buttonSize);
        btnSave.setPreferredSize(buttonSize);
        btnSave.setMaximumSize(buttonSize);

        final ActionListener actionButtonListener = new ActionButtonsAdapter();
        btnSave.addActionListener(actionButtonListener);
        btnCancel.addActionListener(actionButtonListener);
        checkSave.addActionListener(new CheckSaveAdapter());

        mainPanel.add(jLabel1, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.VERTICAL, new Insets(3, 4, 4, 0), 0, 0));
        mainPanel.add(jLabel3, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.VERTICAL, new Insets(0, 4, 4, 0), 0, 0));
        mainPanel.add(inputFolder, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 20, 0));
        mainPanel.add(inputVariable, new GridBagConstraints(3, 0, 1, 1, 1.0, 0.0
                , GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 80, 0));
        mainPanel.add(inputComment, new GridBagConstraints(1, 1, 3, 1, 1.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 4, 4), 0, 0));
        mainPanel.add(jLabel2, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.VERTICAL, new Insets(4, 4, 4, 0), 13, 0));
        picturePanel.add(radioDontChange, new GridBagConstraints(0, 0, 2, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.VERTICAL, new Insets(0, 4, 0, 0), 0, 0));
        picturePanel.add(radioUseLast, new GridBagConstraints(0, 1, 2, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.VERTICAL, new Insets(0, 4, 2, 2), 0, 0));
        picturePanel.add(radioSetNew, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.VERTICAL, new Insets(0, 4, 0, 0), 0, 0));
        picturePanel.add(radioSameAsDocument, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.VERTICAL, new Insets(0, 4, 0, 0), 0, 0));
        picturePanel.add(inputPictureFolder, new GridBagConstraints(1, 4, 1, 1, 1.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.VERTICAL, new Insets(0, -60, 4, 5), 114, 0));
        mainPanel.add(btnPanel, new GridBagConstraints(0, 4, 4, 1, 1.0, 1.0
                , GridBagConstraints.WEST, GridBagConstraints.VERTICAL, new Insets(0, 0, 0, 0), 28, 0));
        mainPanel.add(checkSave, new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.VERTICAL, new Insets(0, 4, 0, -20), 5, 0));
        buttonGroup.add(radioDontChange);
        buttonGroup.add(radioUseLast);
        buttonGroup.add(radioSameAsDocument);
        buttonGroup.add(radioSetNew);
        btnPanel.add(btnCancel, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(4, 11, 4, 7), 0, 0));
        btnPanel.add(btnSave, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(4, 133, 4, 5), 0, 0));
        mainPanel.add(picturePanel, new GridBagConstraints(0, 3, 4, 1, 1.0, 1.0
                , GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        this.setSize(350, 195);
        this.pack();
    }

}
