/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.ti89;

import net.wordrider.utilities.Utils;

import java.awt.image.ColorModel;
import java.awt.image.ImageConsumer;
import java.awt.image.ImageProducer;
import java.awt.image.IndexColorModel;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.Hashtable;

/**
 * @author Vity
 */
public final class TIImageDecoder extends TIFileReader implements ImageProducer {
//    private static final Object TI_COMMENT = "TI_COMMENT";
//    private static final Object TI_VARIABLENAME = "TI_VARIABLENAME";
//    private static final Object TI_FOLDERNAME = "TI_FOLDERNAME";
    private static final Object TI_ORIGFILENAME = "TI_ORIGFILENAME";
    private static final Object TI_ORIGFILEDIR = "TI_ORIGFILEDIR";
    private static final Object TI_IMAGEPATH = "TI_IMAGEPATH";

    /* Since done in-memory, only one consumer */
    private ImageConsumer consumer;
    private final boolean loadError = false;
    private int width = -1;
    private int height = -1;
    private byte[] store;
    private final Hashtable props = new Hashtable(10);
    private final static String[] supportedFormats = {"**TI92P*", "**TI89**"};
    private char sum;

//    public final void setFolderName(final String folderName) {
//        props.put(TI_FOLDERNAME, folderName);
//    }

    public final String getOriginalFileDir() {
        return (String) props.get(TI_ORIGFILEDIR);
    }

    public final String getOriginalFileName() {
        return (String) props.get(TI_ORIGFILENAME);
    }

    public final void setOriginalFileName(String fileName) {
        props.put(TI_ORIGFILENAME, fileName);
    }

    public final void setOriginalFileDir(String origFileDir) {
        props.put(TI_ORIGFILEDIR, origFileDir);
    }


    public final void setImagePath(String imagePath) {
        props.put(TI_IMAGEPATH, imagePath);
    }

    public final String getStringRepresentation() {
        return "&P" + getOriginalFileName();
    }

    public final String getImagePath() {
        return (String) props.get(TI_IMAGEPATH);
    }

    protected final void readContent(DataInputStream stream) throws IOException {
        sum = (char) (stream.readUnsignedByte() + stream.readUnsignedByte()); // bytes for reading
        int tempByte1 = stream.readUnsignedByte();
        int tempByte2 = stream.readUnsignedByte();
        height = (tempByte1 << 8) + tempByte2;
        sum += tempByte1 + tempByte2;
        tempByte1 = stream.readUnsignedByte();
        tempByte2 = stream.readUnsignedByte();
        width = (tempByte1 << 8) + tempByte2;
        sum += tempByte1 + tempByte2;
        if (width % 8 != 0) width += (8 - (width % 8)); // zarovnani na 8
//        System.out.println("height " + height + " width " + width);
        final int compressedWidth = width / 8;
        final byte[] pixels = new byte[height * compressedWidth];
        stream.readFully(pixels);
        final int length = pixels.length;
        store = new byte[length * 8];
        byte bitByte;
        int pixel;
        for (int i = 0, index = 0; i < length; ++i) {
            bitByte = pixels[i];
            sum += (char) (bitByte & 0xFF);//crc counter
            for (int k = 7; k >= 0; k--) {
                pixel = bitByte & (1 << k);
                store[index++] = (pixel != 0) ? (byte) 0xFF : 0;
            }
        }
        sum += stream.readUnsignedByte(); //0xDF
    }

    protected final boolean isSupportedHeader(String header) {
        final int size = supportedFormats.length;
        for (int i = 0; i < size; ++i)
            if (supportedFormats[i].equals(header))
                return true;
        return false;  //implement - call to super class
    }

    public final int getDataType() {
        return DATA_TYPES[DATA_TYPE_IMAGE];  //implement - call to super class
    }

    public final String getFileHeaderType() {
        return supportedFormats[0];  //implement - call to super class
    }

    public final char getCheckSum() {
        return sum;  //implement - call to super class
    }

    /* Format of Ppm file is single pass/frame, w/ complete scan lines in order */
    private static final int hints = (ImageConsumer.TOPDOWNLEFTRIGHT |
            ImageConsumer.COMPLETESCANLINES |
            ImageConsumer.SINGLEPASS |
            ImageConsumer.SINGLEFRAME);


    public final synchronized void addConsumer(ImageConsumer ic) {
        consumer = ic;
        try {
            produce();
        } catch (Exception e) {
            Utils.processException(e);
            if (consumer != null)
                consumer.imageComplete(ImageConsumer.IMAGEERROR);
        }
        consumer = null;
    }
/* If consumer passed to routine is single consumer, return true, else false. */
    public final synchronized boolean isConsumer(ImageConsumer ic) {
        return (ic == consumer);
    }

    /* Disables consumer if currently consuming. */
    public final synchronized void removeConsumer(ImageConsumer ic) {
        if (consumer == ic)
            consumer = null;
    }
/* Production is done by adding consumer. */
    public final void startProduction(ImageConsumer ic) {
        addConsumer(ic);
    }

    public final void requestTopDownLeftRightResend(ImageConsumer ic) {
        // Not needed.  The data is always in this format.
    }

    private void produce() {
        final byte[] bw = {(byte) 0xff, (byte) 0};
        final ColorModel cm = new IndexColorModel(1, 2, bw, bw, bw);
        if (consumer != null) {
            if (loadError) {
                consumer.imageComplete(ImageConsumer.IMAGEERROR);
            } else {
                consumer.setDimensions(width, height);
                consumer.setColorModel(cm);
                consumer.setHints(hints);
                // allocate pixels
                for (int j = 0, offset = 0; j < height; ++j) {
                    consumer.setPixels(0, j, width, 1, cm, store, offset, width);
                    offset += width;
                }
                consumer.setProperties(props);
                consumer.imageComplete(ImageConsumer.STATICIMAGEDONE);
                //  store = null;
            }
        }
    }

}
